@@ -28,7 +28,7 @@
 
 #if 0
 static const char *version =
-"cardinfo.c 1.15 1996/11/26 07:25:29 (David Hinds)";
+"cardinfo.c 1.16 1997/03/21 07:24:30 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -92,6 +92,7 @@ static int open_dev(dev_t dev)
 
 static void print_status(status_t *status)
 {
+    printf("  Function %d:\n", status->Function);
     printf("  ");
     if (status->CardState & CS_EVENT_CARD_DETECT)
 	printf("card present");
@@ -229,9 +230,11 @@ static int do_cmd(int fd, int cmd)
     switch (cmd) {
 	
     case C_STATUS:
-	ret = ioctl(fd, DS_GET_STATUS, &status);
-	if (ret == 0)
-	    print_status(&status);
+	for (i = 0; i < 4; i++) {
+	    status.Function = i;
+	    if (ioctl(fd, DS_GET_STATUS, &status) == 0)
+		print_status(&status);
+	}
 	break;
 	
     case C_CONFIG:
