@@ -45,7 +45,7 @@
 #include "cardmgr.h"
 
 static const char *version =
-    "cardmgr.c 1.84 1996/12/07 05:09:35 (David Hinds)";
+    "cardmgr.c 1.87 1997/03/21 17:08:22 (David Hinds)";
 
 /*====================================================================*/
 
@@ -87,9 +87,9 @@ static struct socket_info_t socket[MAX_SOCKS];
 
 /* Default path for config file, device scripts */
 #ifdef ETC
-static char *config = ETC;
+static char *configpath = ETC;
 #else
-static char *config = "/etc/pcmcia";
+static char *configpath = "/etc/pcmcia";
 #endif
 
 /* Default path for pid file */
@@ -107,6 +107,9 @@ static int be_quiet = 0;
 /* If set, use modprobe instead of insmod */
 static int do_modprobe = 0;
 
+/* If set, configure already inserted cards, then exit */
+static int one_pass = 0;
+
 /* most recent signal */
 static int caught_signal = 0;
 
@@ -297,6 +300,7 @@ static card_info_t *lookup_card(int ns)
     card_info_t *card;
     ds_ioctl_arg_t arg;
     cistpl_vers_1_t *vers = NULL;
+    cistpl_manfid_t manfid = { 0, 0 };
     int i, ret, match;
     int has_cis = 0;
 
@@ -304,8 +308,11 @@ static card_info_t *lookup_card(int ns)
     ret = ioctl(s->fd, DS_VALIDATE_CIS, &arg);
     has_cis = ((ret == 0) && (arg.cisinfo.Chains > 0));
     
-    /* Try to read VERS_1 tuple */
+    /* Try to read VERS_1, MANFID tuples */
     if (has_cis) {
+	if (get_tuple(ns, CISTPL_MANFID, &arg) == 0)
+	    memcpy(&manfid, &arg.tuple_parse.parse.manfid,
+		   sizeof(manfid));
 	if (get_tuple(ns, CISTPL_VERS_1, &arg) == 0)
 	    vers = &arg.tuple_parse.parse.version_1;
 
@@ -329,6 +336,12 @@ static card_info_t *lookup_card(int ns)
 		    break;
 		match = 1;
 		break;
+
+	    case MANFID_IDENT:
+		if ((manfid.manf == card->manfid.manf) &&
+		    (manfid.card == card->manfid.card))
+		    match = 1;
+		break;
 		
 	    case TUPLE_IDENT:
 		arg.tuple.DesiredTuple = card->tuple.code;
@@ -371,6 +384,7 @@ static card_info_t *lookup_card(int ns)
     
     if (vers || (!blank_card)) {
 	syslog(LOG_INFO, "unsupported card in socket %d", ns);
+	if (one_pass) return NULL;
 	beep(BEEP_TIME, BEEP_ERR);
 	if (vers) {
 	    char v[256] = "";
@@ -381,9 +395,11 @@ static card_info_t *lookup_card(int ns)
 	}
 	else
 	    syslog(LOG_INFO, "no version info");
+	if (manfid.manf != 0)
+	    syslog(LOG_INFO, "manfid: 0x%04x, 0x%04x",
+		   manfid.manf, manfid.card);
 	return NULL;
-    }
-    else {
+    } else {
 	card = blank_card;
 	syslog(LOG_INFO, "socket %d: %s", ns, card->name);
 	beep(BEEP_TIME, BEEP_WARN);
@@ -395,9 +411,9 @@ static card_info_t *lookup_card(int ns)
 
 static void load_config(void)
 {
-    char fn[128];
-    sprintf(fn, "%s/config", config);
-    if (parse_configfile(fn) != 0)
+    if (chdir(configpath) != 0)
+	syslog(LOG_INFO, "chdir to %s failed: %m", configpath);
+    if (parse_configfile("config") != 0)
 	exit(EXIT_FAILURE);
     if (root_device == NULL)
 	syslog(LOG_INFO, "no device drivers defined");
@@ -503,7 +519,7 @@ static int execute_on_dev(char *action, char *class, char *dev)
     char msg[128], cmd[512];
 
     sprintf(msg, "%s cmd", action);
-    sprintf(cmd, "%s/%s %s %s", config, class, action, dev);
+    sprintf(cmd, "./%s %s %s", class, action, dev);
     return execute(msg, cmd);
 }
 
@@ -546,7 +562,7 @@ static int install_module(char *mod, char *opts)
     ml->usage++;
     if (ml->usage == 1) {
 	if (do_modprobe) {
-	    sprintf(cmd, "/sbin/modprobe %s", mod);
+	    sprintf(cmd, "modprobe %s", mod);
 	} else {
 	    if (strchr(mod, '/') != NULL)
 		sprintf(path, "%s/%s.o", modpath, mod);
@@ -556,7 +572,7 @@ static int install_module(char *mod, char *opts)
 		syslog(LOG_INFO, "module %s not available", path);
 		return -1;
 	    }
-	    sprintf(cmd, "/sbin/insmod %s", path);
+	    sprintf(cmd, "insmod %s", path);
 	}
 	if (opts) {
 	    strcat(cmd, " ");
@@ -584,7 +600,7 @@ static void remove_module(char *mod)
 		s = mod;
 	    else
 		s++;
-	    sprintf(cmd, "/sbin/rmmod %s", s);
+	    sprintf(cmd, "rmmod %s", s);
 	    execute("rmmod", cmd);
 	}
     }
@@ -714,9 +730,12 @@ static void do_insert(int sn)
 	strcpy((char *)bind->dev_info, (char *)dev[i]->dev_info);
 	bind->function = card->dev_fn[i];
 	if (ioctl(s->fd, DS_BIND_REQUEST, bind) != 0) {
-	    syslog(LOG_INFO, "bind '%s' to socket %d failed: %m",
-		   (char *)bind->dev_info, sn);
-	    if (errno != EBUSY) {
+	    if (errno == EBUSY) {
+		syslog(LOG_INFO, "'%s' already bound to socket %d",
+		       (char *)bind->dev_info, sn);
+	    } else {
+		syslog(LOG_INFO, "bind '%s' to socket %d failed: %m",
+		       (char *)bind->dev_info, sn);
 		beep(BEEP_TIME, BEEP_ERR);
 		write_stab();
 		return;
@@ -945,6 +964,16 @@ static void adjust_resources(void)
 
 static int cleanup_files = 0;
 
+static void fork_now(void)
+{
+    int ret;
+    if ((ret = fork()) > 0) exit(0);
+    if (ret == -1)
+	syslog(LOG_INFO, "forking: %m");
+    if (setsid() < 0)
+	syslog(LOG_INFO, "detaching from tty: %m");
+}
+
 static void done(void)
 {
     syslog(LOG_INFO, "exiting");
@@ -1040,14 +1069,15 @@ static int init_sockets(void)
 void main(int argc, char *argv[])
 {
     int optch, errflg;
-    int i, max_fd, ret, event;
-    int verbose = 0;
+    int i, max_fd, ret, event, pass;
+    int verbose = 0, delay_fork = 0;
+    struct timeval tv;
     extern char *optarg;
     fd_set fds;
     FILE *f;
 
     errflg = 0;
-    while ((optch = getopt(argc, argv, "qvdc:m:p:s:")) != -1) {
+    while ((optch = getopt(argc, argv, "qdvofc:m:p:s:")) != -1) {
 	switch (optch) {
 	case 'q':
 	    be_quiet = 1; break;
@@ -1055,8 +1085,12 @@ void main(int argc, char *argv[])
 	    do_modprobe = 1; break;
 	case 'v':
 	    verbose = 1; break;
+	case 'o':
+	    one_pass = 1; break;
+	case 'f':
+	    delay_fork = 1; break;
 	case 'c':
-	    config = strdup(optarg); break;
+	    configpath = strdup(optarg); break;
 	case 'm':
 	    modpath = strdup(optarg); break;
 	case 'p':
@@ -1068,9 +1102,9 @@ void main(int argc, char *argv[])
 	}
     }
     if (errflg || (optind < argc)) {
-	fprintf(stderr, "usage: %s [-q] [-v] [-d] [-c config] "
-		"[-m modpath]\n               [-p pidfile] "
-		"[s stabfile]\n", argv[0]);
+	fprintf(stderr, "usage: %s [-q] [-v] [-d] [-o] [-f] "
+		"[-c configpath] [-m modpath]\n               "
+		"[-p pidfile] [-s stabfile]\n", argv[0]);
 	exit(EXIT_FAILURE);
     }
 
@@ -1084,11 +1118,8 @@ void main(int argc, char *argv[])
 #endif
 
 #ifndef DEBUG
-    if ((ret = fork()) > 0) exit(0);
-    if (ret == -1)
-	syslog(LOG_INFO, "forking: %m");
-    if (setsid() < 0)
-	syslog(LOG_INFO, "detaching from tty: %m");
+    if (!delay_fork && !one_pass)
+	fork_now();
 #endif
     
     syslog(LOG_INFO, "starting");
@@ -1135,13 +1166,17 @@ void main(int argc, char *argv[])
 
     for (i = max_fd = 0; i < sockets; i++)
 	max_fd = (socket[i].fd > max_fd) ? socket[i].fd : max_fd;
+
+    /* First select() call: poll, don't wait */
+    tv.tv_sec = tv.tv_usec = 0;
     
-    for (;;) {
+    for (pass = 0; ; pass++) {
 	FD_ZERO(&fds);
 	for (i = 0; i < sockets; i++)
 	    FD_SET(socket[i].fd, &fds);
 
-	while ((ret = select(max_fd+1, &fds, NULL, NULL, NULL)) < 0) {
+	while ((ret = select(max_fd+1, &fds, NULL, NULL,
+			     ((pass == 0) ? &tv : NULL))) < 0) {
 	    if (errno == EINTR)
 		handle_signal();
 	    else {
@@ -1191,5 +1226,11 @@ void main(int argc, char *argv[])
 	    }
 	    
 	}
+
+	if (one_pass)
+	    exit(EXIT_SUCCESS);
+	if (delay_fork)
+	    fork_now();
+	
     } /* repeat */
 }
