@@ -1,4 +1,4 @@
-/* cardmgr.h 1.22 1995/12/16 23:30:03 (David Hinds) */
+/* cardmgr.h 1.23 1997/03/21 07:24:48 (David Hinds) */
 
 #define MAX_SOCKS	8
 #define MAX_FUNCTIONS	4
@@ -13,6 +13,11 @@ typedef struct func_ident_t {
     u_char		funcid;
 } func_ident_t;
 
+typedef struct manfid_ident_t {
+    u_short		manf;
+    u_short		card;
+} manfid_ident_t;
+
 typedef struct vers_ident_t {
     int			ns;
     char		*pi[4];
@@ -37,9 +42,10 @@ typedef struct device_info_t {
 typedef struct card_info_t {
     char		*name;
     enum {
-	VERS_1_IDENT=1, TUPLE_IDENT, FUNC_IDENT, BLANK_IDENT
+	VERS_1_IDENT=1, MANFID_IDENT, TUPLE_IDENT, FUNC_IDENT, BLANK_IDENT
     } ident_type;
     vers_ident_t	vers;
+    manfid_ident_t	manfid;
     tuple_ident_t	tuple;
     func_ident_t	func;
     int			functions;
