@@ -4,7 +4,7 @@
 
     Written by David Hinds, dhinds@allegro.stanford.edu
 
-    scsi_info.c 1.4 1996/05/08 16:52:07
+    scsi_info.c 1.5 1996/11/26 07:25:29
     
 ======================================================================*/
 
@@ -17,8 +17,8 @@
 #include <sys/ioctl.h>
 #include <dirent.h>
 
-#include "version.h"
-#include "config.h"
+#include <linux/version.h>
+#include <linux/config.h>
 
 #include <linux/major.h>
 #define VERSION(v,p,s) (((v)<<16)+(p<<8)+s)
