@@ -4,7 +4,7 @@
 
     Written by David Hinds, dhinds@allegro.stanford.edu
 
-    ftl_format.c 1.2 1995/12/02 07:56:55
+    ftl_format.c 1.3 1996/11/26 07:24:56
 
 ======================================================================*/
 
@@ -20,10 +20,10 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 
-#include "cs_types.h"
-#include "cs.h"
-#include "ftl.h"
-#include "memory.h"
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/ftl.h>
+#include <pcmcia/memory.h>
 
 /*====================================================================*/
 
