@@ -1,5 +1,5 @@
 /*
- * ciscode.h 1.7 1996/12/23 06:33:03 (David Hinds)
+ * ciscode.h 1.8 1997/01/26 06:27:54 (David Hinds)
  */
 
 #ifndef _LINUX_CISCODE_H
@@ -41,6 +41,9 @@
 #define MANFID_SMC			0x0108
 #define PRODID_SMC_ETHER		0x0105
 
+#define MANFID_SOCKET			0x0104
+#define PRODID_SOCKET_DUAL_RS232	0x0006
+
 #define MANFID_SUNDISK			0x0045
 
 #endif /* _LINUX_CISCODE_H */
