@@ -1,5 +1,5 @@
 /*
- * k_compat.h 1.18 1996/11/20 07:49:37 (David Hinds)
+ * k_compat.h 1.28 1997/03/12 19:23:31 (David Hinds)
  */
 
 #ifndef _LINUX_K_COMPAT_H
@@ -15,6 +15,10 @@
 #define U_FS_SIZE_T unsigned long
 #endif
 
+#if (LINUX_VERSION_CODE < VERSION(2,1,25))
+#define net_device_stats enet_statistics
+#endif
+
 #if (LINUX_VERSION_CODE < VERSION(1,3,0))
 
 #define RUN_AT(x) (x)
@@ -71,6 +75,10 @@ char kernel_version[] = UTS_RELEASE;
 
 #else /* 1.3.38 */
 
+#if (LINUX_VERSION_CODE > VERSION(2,1,16))
+#define AUTOCONF_INCLUDED
+#define EXPORT_SYMTAB
+#endif
 #ifdef CONFIG_MODVERSIONS
 #define MODVERSIONS 1
 #if (LINUX_VERSION_CODE >= VERSION(1,3,40))
@@ -81,23 +89,32 @@ char kernel_version[] = UTS_RELEASE;
 
 #endif /* 1.3.38 */
 
+#if (LINUX_VERSION_CODE < VERSION(2,1,18))
+#define MOD_USE_COUNT mod_use_count_
+#define MODULE_PARM(a,b) extern int __bogus_decl
+#else
+#define MOD_USE_COUNT __this_module.usecount
+#endif
+
 #if (LINUX_VERSION_CODE >= VERSION(1,3,44))
 #define NEW_MULTICAST
 #endif /* 1.3.44 */
 
 #if (LINUX_VERSION_CODE >= VERSION(1,3,70))
-#define FREE_IRQ(i) free_irq(i, NULL)
-#define REQUEST_IRQ(i,h,f,n) request_irq(i,h,f,n,NULL)
+#define FREE_IRQ(i,d) free_irq(i, d)
+#define REQUEST_IRQ(i,h,f,n,d) request_irq(i,h,f,n,d)
 #define IRQ(a,b,c) (a,b,c)
 #else
-#define FREE_IRQ(i) free_irq(i)
-#define REQUEST_IRQ(i,h,f,n) request_irq(i,h,f,n)
+#define SA_SHIRQ 0
+#define FREE_IRQ(i,d) free_irq(i)
+#define REQUEST_IRQ(i,h,f,n,d) request_irq(i,h,f,n)
 #define IRQ(a,b,c) (a,c)
 #endif
 
 #if (LINUX_VERSION_CODE < VERSION(2,1,0))
 #define copy_from_user memcpy_fromfs
 #define copy_to_user memcpy_tofs
+
 #if (LINUX_VERSION_CODE >= VERSION(1,3,0))
 /* This is evil... throw away the built-in get_user in 1.3, 2.0 */
 #include <asm/segment.h>
@@ -108,10 +125,16 @@ char kernel_version[] = UTS_RELEASE;
  (sizeof(*ptr) == 2) ? put_fs_word(x, ptr) : \
  put_fs_byte(x, ptr))
 #endif
+
+#ifdef __alpha__
+#define get_user(x, ptr) __get_user(x, ptr)
+#else
 #define get_user(x, ptr) \
 ((sizeof(*ptr) == 4) ? (x = get_fs_long(ptr)) : \
  (sizeof(*ptr) == 2) ? (x = get_fs_word(ptr)) : \
  (x = get_fs_byte(ptr)))
+#endif
+
 #else /* 2.1.X */
 #include <asm/uaccess.h>
 #endif
