@@ -45,6 +45,9 @@
 #include <pcmcia/ciscode.h>
 #include <pcmcia/ds.h>
 
+/* Include code for transceiver autodetection? */
+/*#define AUTODETECT*/
+
 /* To minimize the size of the driver source I only define operating
    constants if they are used several times.  You'll need the manual
    if you want to understand driver details. */
@@ -92,18 +95,21 @@ enum RxFilter {
 
 struct el3_private {
     dev_node_t node;
+    struct net_device_stats stats;
+#ifdef AUTODETECT
     int probe_port;
     struct timer_list check;
     int flipped;
-    struct enet_statistics stats;
+#endif
 };
 
 static char *if_names[] = { "Auto", "10baseT", "10base2", "AUI" };
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"3c589_cs.c 1.86 1996/12/21 19:30:42 (David Hinds)";
+"3c589_cs.c $Revision: 1.88 $ $Date: 1997/02/08 07:57:48 $ (David Hinds)";
 #endif
 
 #ifdef TC589_DEBUG
@@ -112,6 +118,8 @@ static int tc589_debug = TC589_DEBUG;
 static int tc589_debug = 3;
 #endif
 
+MODULE_PARM(tc589_debug, "i");
+
 /*====================================================================*/
 
 /* Parameters that can be set with 'insmod' */
@@ -122,6 +130,9 @@ static int if_port = 1;
 /* Bit map of interrupts to choose from */
 static u_long irq_mask = 0xdeb8;
 
+MODULE_PARM(if_port, "i");
+MODULE_PARM(irq_mask, "i");
+
 /*====================================================================*/
 
 static void tc589_config(dev_link_t *link);
@@ -131,13 +142,15 @@ static int tc589_event(event_t event, int priority,
 
 static ushort read_eeprom(short ioaddr, int index);
 static void tc589_reset(struct device *dev);
+#ifdef AUTODETECT
 static void check_if_port(u_long i);
+#endif
 static int el3_config(struct device *dev, struct ifmap *map);
 static int el3_open(struct device *dev);
 static int el3_start_xmit(struct sk_buff *skb, struct device *dev);
 static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs);
 static void update_stats(int addr, struct device *dev);
-static struct enet_statistics *el3_get_stats(struct device *dev);
+static struct net_device_stats *el3_get_stats(struct device *dev);
 static int el3_rx(struct device *dev);
 static int el3_close(struct device *dev);
 
@@ -638,7 +651,9 @@ static int el3_config(struct device *dev, struct ifmap *map)
 
 static int el3_open(struct device *dev)
 {
+#ifdef AUTODETECT
     struct el3_private *lp = (struct el3_private *)dev->priv;
+#endif
     int ioaddr = dev->base_addr;
     dev_link_t *link;
     
@@ -655,9 +670,11 @@ static int el3_open(struct device *dev)
     irq2dev_map[dev->irq] = dev;
     
     dev->interrupt = 0; dev->tbusy = 0; dev->start = 1;
+#ifdef AUTODETECT
     lp->probe_port = 15; lp->flipped = 0;
     lp->check.function = &check_if_port;
     lp->check.data = (u_long)dev;
+#endif
     tc589_reset(dev);
     
     if (tc589_debug > 3)
@@ -672,8 +689,10 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
     struct el3_private *lp = (struct el3_private *)dev->priv;
     int ioaddr = dev->base_addr;
 
+#ifdef AUTODETECT
     if (lp->probe_port)
 	check_if_port((u_long)dev);
+#endif
     
     /* Transmitter timeout, serious problems. */
     if (dev->tbusy) {
@@ -691,18 +710,19 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
 	outw(TxEnable, ioaddr + EL3_CMD);
 	dev->tbusy = 0;
     }
-    
+
+#if (LINUX_VERSION_CODE < VERSION(2,1,25))
     if (skb == NULL) {
 	dev_tint(dev);
 	return 0;
     }
-    
     if (skb->len <= 0)
 	return 0;
+#endif
     
     if (tc589_debug > 4) {
 	printk(KERN_DEBUG "%s: el3_start_xmit(length = %ld) called, "
-	       "status %4.4x.\n", dev->name, skb->len,
+	       "status %4.4x.\n", dev->name, (long)skb->len,
 	       inw(ioaddr + EL3_STATUS));
     }
 #ifdef PCMCIA_DEBUG
@@ -730,6 +750,9 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
 	printk(KERN_NOTICE "%s: transmitter access conflict.\n",
 	       dev->name);
     else {
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+	lp->stats.tx_bytes += skb->len;
+#endif
 	/* Put out the doubleword header... */
 	outw(skb->len, ioaddr + TX_FIFO);
 	outw(0x00, ioaddr + TX_FIFO);
@@ -759,10 +782,12 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
 	}
     }
 
+#ifdef AUTODETECT
     if (lp->probe_port && (lp->check.next == NULL)) {
 	lp->check.expires = RUN_AT(HZ/20);
 	add_timer(&lp->check);
     }
+#endif
     
     return 0;
 }
@@ -775,26 +800,22 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
     int ioaddr, status;
     int i = 0;
     
-    if (dev == NULL) {
-	printk(KERN_DEBUG "el3_interrupt(): irq %d for unknown"
-	       " device.\n", irq);
+    if (dev == NULL)
 	return;
-    }
     lp = (struct el3_private *)dev->priv;
-    
+    ioaddr = dev->base_addr;
+
+#ifdef PCMCIA_DEBUG
     if (dev->interrupt) {
 	printk(KERN_NOTICE "%s: re-entering the interrupt handler.\n",
 	       dev->name);
 	return;
     }
     dev->interrupt = 1;
-    
-    ioaddr = dev->base_addr;
-    status = inw(ioaddr + EL3_STATUS);
-    
     if (tc589_debug > 4)
 	printk(KERN_DEBUG "%s: interrupt, status %4.4x.\n",
-	       dev->name, status);
+	       dev->name, inw(ioaddr + EL3_STATUS));
+#endif
     
     while ((status = inw(ioaddr + EL3_STATUS)) &
 	(IntLatch | RxComplete | StatsFull)) {
@@ -845,24 +866,27 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 	}
 	/* Acknowledge the IRQ. */
 	outw(AckIntr | IntReq | IntLatch, ioaddr + EL3_CMD);
-	
+
+#ifdef AUTODETECT
 	if (lp->probe_port)
 	    check_if_port((u_long)dev);
-    
-    }
-    
-    if (tc589_debug > 4) {
-	printk(KERN_DEBUG "%s: exiting interrupt, status %4.4x.\n",
-	       dev->name, inw(ioaddr + EL3_STATUS));
+#endif
     }
 
+#ifdef PCMCIA_DEBUG
+    if (tc589_debug > 4)
+	printk(KERN_DEBUG "%s: exiting interrupt, status %4.4x.\n",
+	       dev->name, inw(ioaddr + EL3_STATUS));
+    
     dev->interrupt = 0;
+#endif
     return;
 }
 
 /* Check for a transceiver mismatch.  We should get carrier errors when
    there is a mismatch. */
 
+#ifdef AUTODETECT
 static void check_if_port(u_long i)
 {
     struct device *dev = (struct device *)i;
@@ -912,9 +936,9 @@ static void check_if_port(u_long i)
     }
     in_check = 0;
 }
+#endif /* AUTODETECT */
 
-
-static struct enet_statistics *el3_get_stats(struct device *dev)
+static struct net_device_stats *el3_get_stats(struct device *dev)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
     unsigned long flags;
@@ -999,6 +1023,9 @@ static int el3_rx(struct device *dev)
 		netif_rx(skb);
 		outw(RxDiscard, ioaddr + EL3_CMD); /* Pop top Rx packet. */
 		lp->stats.rx_packets++;
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+		lp->stats.rx_bytes += skb->len;
+#endif
 		continue;
 	    } else if (tc589_debug)
 		printk(KERN_DEBUG "%s: couldn't allocate a sk_buff of"
