@@ -50,8 +50,9 @@
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"aha152x_cs.c 1.24 1996/12/21 19:30:43 (David Hinds)";
+"aha152x_cs.c 1.27 1997/03/16 20:03:23 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -70,6 +71,13 @@ static int synchronous = 0;
 /* Time to pause during bus reset, in jiffies */
 static int reset_delay = 100;
 
+MODULE_PARM(irq_mask, "i");
+MODULE_PARM(host_id, "i");
+MODULE_PARM(reconnect, "i");
+MODULE_PARM(parity, "i");
+MODULE_PARM(synchronous, "i");
+MODULE_PARM(reset_delay, "i");
+
 /*====================================================================*/
 
 typedef struct scsi_info_t {
@@ -235,7 +243,11 @@ static void aha152x_config_x(dev_link_t *link)
     }
 
     /* Configure card */
-    driver_template.usage_count = &mod_use_count_;
+#if (LINUX_VERSION_CODE >= VERSION(2,1,23))
+    driver_template.module = &__this_module;
+#else
+    driver_template.usage_count = &MOD_USE_COUNT;
+#endif
     link->state |= DEV_CONFIG;
 
     do {
@@ -257,7 +269,7 @@ static void aha152x_config_x(dev_link_t *link)
 		(link->io.BasePort1 < 0xffff)) {
 		link->conf.ConfigIndex = parse.cftable_entry.index;
 		i = CardServices(RequestIO, link->handle, &link->io);
-		if (i == CS_SUCCESS) break;
+		if (i != CS_IN_USE) break;
 	    }
 	    i = CardServices(GetNextTuple, handle, &tuple);
 	}
@@ -362,7 +374,11 @@ static void aha152x_release(u_long arg)
 	printk(KERN_DEBUG "aha152x_release(0x%p)\n", link);
 #endif
 
+#if (LINUX_VERSION_CODE < VERSION(2,1,23))
     if (*driver_template.usage_count != 0) {
+#else
+    if (driver_template.module->usecount != 0) {
+#endif
 #ifdef PCMCIA_DEBUG
 	if (pc_debug)
 	    printk(KERN_DEBUG "aha152x_cs: release postponed, "
