@@ -6,11 +6,9 @@
     
 ======================================================================*/
 
-#ifdef MODULE
-#include <linux/autoconf.h>
-#include <linux/module.h>
-#include <linux/version.h>
-#endif
+#include <pcmcia/config.h>
+#define __NO_VERSION__
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/string.h>
@@ -20,14 +18,13 @@
 #include <linux/sched.h>
 #include <linux/timer.h>
 
-#include "cs_types.h"
-#include "ss.h"
-
 #define IN_CARD_SERVICES
-#include "cs.h"
-#include "bulkmem.h"
+#include <pcmcia/cs_types.h>
+#include <pcmcia/ss.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/bulkmem.h>
+#include <pcmcia/cistpl.h>
 #include "cs_internal.h"
-#include "cistpl.h"
 
 /*======================================================================
 
@@ -76,7 +73,7 @@ static int do_mtd_request(memory_handle_t handle, mtd_request_t *req,
 	    break;
 	}
 	if (current->signal & ~current->blocked)
-	    printk("cs: do_mtd_request interrupted!\n");
+	    printk(KERN_NOTICE "cs: do_mtd_request interrupted!\n");
     }
     if (tries == 20)
 	ret = CS_GENERAL_FAILURE;
@@ -97,7 +94,7 @@ static void insert_queue(erase_busy_t *head, erase_busy_t *entry)
 {
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	printk("Adding 0x%p to queue 0x%p\n", entry, head);
+	printk(KERN_DEBUG "Adding 0x%p to queue 0x%p\n", entry, head);
 #endif
     entry->next = head;
     entry->prev = head->prev;
@@ -109,7 +106,7 @@ static void remove_queue(erase_busy_t *entry)
 {
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	printk("Unqueueing 0x%p\n", entry);
+	printk(KERN_DEBUG "Unqueueing 0x%p\n", entry);
 #endif
     entry->next->prev = entry->prev;
     entry->prev->next = entry->next;
@@ -125,7 +122,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	printk("Trying erase request 0x%p...\n", busy);
+	printk(KERN_DEBUG "Trying erase request 0x%p...\n", busy);
 #endif
     if (busy->next)
 	remove_queue(busy);
@@ -140,7 +137,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
     if (ret == CS_BUSY) {
 #ifdef PCMCIA_DEBUG
 	if (pc_debug > 1)
-	    printk("  Status = %ld, requeueing.\n", req.Status);
+	    printk(KERN_DEBUG "  Status = %ld, requeueing.\n", req.Status);
 #endif
 	switch (req.Status) {
 	case MTD_WAITREQ:
@@ -151,7 +148,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
 	case MTD_WAITRDY:
 	    if (req.Status == MTD_WAITRDY)
 		insert_queue(&s->erase_busy, busy);
-	    busy->timeout.expires = req.Timeout/10;
+	    busy->timeout.expires = RUN_AT(req.Timeout*HZ/1000);
 	    add_timer(&busy->timeout);
 	    break;
 	}
@@ -160,7 +157,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
 	/* update erase queue status */
 #ifdef PCMCIA_DEBUG
 	if (pc_debug > 1)
-	    printk("  Ret = %d\n", ret);
+	    printk(KERN_DEBUG "  Ret = %d\n", ret);
 #endif
 	switch (ret) {
 	case CS_SUCCESS:
@@ -191,7 +188,7 @@ void retry_erase_list(erase_busy_t *list, u_long cause)
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	printk("Rescanning erase queue list 0x%p\n", list);
+	printk(KERN_DEBUG "Rescanning erase queue list 0x%p\n", list);
 #endif
     if (list->next == list)
 	return;
@@ -211,7 +208,7 @@ static void handle_erase_timeout(u_long arg)
 {
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	printk("Erase timeout for entry 0x%lx\n", arg);
+	printk(KERN_DEBUG "Erase timeout for entry 0x%lx\n", arg);
 #endif
     retry_erase((erase_busy_t *)arg, MTD_REQ_TIMEOUT);
 }
@@ -320,25 +317,6 @@ int MTDHelperEntry(int func, void *a1, void *a2)
     
 ======================================================================*/
 
-static int read_tuple(client_handle_t handle, cisdata_t code, void *parse)
-{
-    tuple_t tuple;
-    cisdata_t buf[255];
-    int ret;
-    
-    tuple.DesiredTuple = code;
-    tuple.Attributes = 0;
-    ret = CardServices(GetFirstTuple, handle, &tuple, NULL);
-    if (ret != CS_SUCCESS) return ret;
-    tuple.TupleData = buf;
-    tuple.TupleOffset = 0;
-    tuple.TupleDataMax = sizeof(buf);
-    ret = CardServices(GetTupleData, handle, &tuple, NULL);
-    if (ret != CS_SUCCESS) return ret;
-    ret = CardServices(ParseTuple, handle, &tuple, parse);
-    return ret;
-}
-
 static void setup_regions(client_handle_t handle, int attr,
 			  memory_handle_t *list)
 {
@@ -351,8 +329,8 @@ static void setup_regions(client_handle_t handle, int attr,
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
-	printk ("cs: setup_regions(0x%p, %d, 0x%p)\n",
-		handle, attr, list);
+	printk(KERN_DEBUG "cs: setup_regions(0x%p, %d, 0x%p)\n",
+	       handle, attr, list);
 #endif
     code = (attr) ? CISTPL_DEVICE_A : CISTPL_DEVICE;
     if (read_tuple(handle, code, &device) != CS_SUCCESS)
@@ -361,7 +339,7 @@ static void setup_regions(client_handle_t handle, int attr,
     has_jedec = (read_tuple(handle, code, &jedec) == CS_SUCCESS);
     if (has_jedec && (device.ndev != jedec.nid)) {
 #ifdef PCMCIA_DEBUG
-	printk("cs: Device info does not match JEDEC info.\n");
+	printk(KERN_DEBUG "cs: Device info does not match JEDEC info.\n");
 #endif
 	has_jedec = 0;
     }
@@ -369,14 +347,15 @@ static void setup_regions(client_handle_t handle, int attr,
     has_geo = (read_tuple(handle, code, &geo) == CS_SUCCESS);
     if (has_geo && (device.ndev != geo.ngeo)) {
 #ifdef PCMCIA_DEBUG
-	printk("cs: Device info does not match geometry tuple.\n");
+	printk(KERN_DEBUG "cs: Device info does not match geometry tuple.\n");
 #endif
 	has_geo = 0;
     }
     
     offset = 0;
     for (i = 0; i < device.ndev; i++) {
-	if (device.dev[i].type != CISTPL_DTYPE_NULL) {
+	if ((device.dev[i].type != CISTPL_DTYPE_NULL) &&
+	    (device.dev[i].size != 0)) {
 	    r = kmalloc(sizeof(*r), GFP_KERNEL);
 	    r->region_magic = REGION_MAGIC;
 	    r->state = 0;
@@ -435,7 +414,7 @@ int get_first_region(client_handle_t handle, region_info_t *rgn)
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
     
-    if ((handle->Attributes & INFO_DRIVER_SERVICES) &&
+    if ((handle->Attributes & INFO_MASTER_CLIENT) &&
 	(!(s->state & SOCKET_REGION_INFO))) {
 	setup_regions(handle, 0, &s->c_region);
 	setup_regions(handle, 1, &s->a_region);
@@ -475,8 +454,8 @@ int register_mtd(client_handle_t handle, mtd_reg_t *reg)
 	list = s->c_region;
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
-	printk ("cs: register_mtd(0x%p, '%s', 0x%lx)\n",
-		handle, (char *)handle->dev_info, reg->Offset);
+	printk(KERN_DEBUG "cs: register_mtd(0x%p, '%s', 0x%lx)\n",
+	       handle, (char *)handle->dev_info, reg->Offset);
 #endif
     while (list) {
 	if (list->info.CardOffset == reg->Offset) break;
@@ -567,8 +546,8 @@ int open_memory(client_handle_t *handle, open_mem_t *open)
 	*handle = (client_handle_t)region;
 #ifdef PCMCIA_DEBUG
 	if (pc_debug)
-	    printk ("cs: open_memory(0x%p, 0x%lx) = 0x%p\n",
-		    handle, open->Offset, region);
+	    printk(KERN_DEBUG "cs: open_memory(0x%p, 0x%lx) = 0x%p\n",
+		   handle, open->Offset, region);
 #endif
 	return CS_SUCCESS;
     }
@@ -588,7 +567,7 @@ int close_memory(memory_handle_t handle)
 {
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
-	printk ("cs: close_memory(0x%p)\n", handle);
+	printk(KERN_DEBUG "cs: close_memory(0x%p)\n", handle);
 #endif
     if (CHECK_REGION(handle))
 	return CS_BAD_HANDLE;
