@@ -6,9 +6,9 @@
     
 ======================================================================*/
 
-#include "config.h"
+#include <pcmcia/config.h>
 #define __NO_VERSION__
-#include "k_compat.h"
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/string.h>
@@ -20,13 +20,13 @@
 #include <linux/sched.h>
 #include <asm/io.h>
 
-#include "cs_types.h"
-#include "ss.h"
-#include "cs.h"
-#include "rsrc_mgr.h"
-#include "bulkmem.h"
+#include <pcmcia/cs_types.h>
+#include <pcmcia/ss.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/bulkmem.h>
+#include <pcmcia/cistpl.h>
 #include "cs_internal.h"
-#include "cistpl.h"
+#include "rsrc_mgr.h"
 
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
 
