@@ -39,13 +39,16 @@ static int handle_apm_event(apm_event_t event);
 
 #ifdef PCMCIA_DEBUG
 int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static const char *version =
-"cs.c 1.141 1996/12/21 19:30:44 (David Hinds)";
+"cs.c 1.145 1997/03/16 20:03:53 (David Hinds)";
 #endif
 
 static const char *release =
     "Linux PCMCIA Card Services " CS_RELEASE;
 
+#if (LINUX_VERSION_CODE <= VERSION(2,1,17))
+
 #undef CONFIG_MODVERSIONS
 static struct symbol_table cs_symtab = {
 #include <linux/symtab_begin.h>
@@ -60,6 +63,16 @@ static struct symbol_table cs_symtab = {
 #include <linux/symtab_end.h>
 };
 
+#else
+
+#define register_symtab(n)
+EXPORT_SYMBOL(register_ss_entry);
+EXPORT_SYMBOL(unregister_ss_entry);
+EXPORT_SYMBOL(CardServices);
+EXPORT_SYMBOL(MTDHelperEntry);
+
+#endif
+
 /*====================================================================*/
 
 /* Parameters that can be set with 'insmod' */
@@ -79,6 +92,17 @@ static int cis_speed		= 300;	/* ns */
 /* Access speed for IO windows */
 static int io_speed		= 0;	/* ns */
 
+MODULE_PARM(setup_delay, "i");
+MODULE_PARM(resume_delay, "i");
+MODULE_PARM(shutdown_delay, "i");
+MODULE_PARM(vcc_settle, "i");
+MODULE_PARM(reset_time, "i");
+MODULE_PARM(unreset_delay, "i");
+MODULE_PARM(unreset_check, "i");
+MODULE_PARM(unreset_limit, "i");
+MODULE_PARM(cis_speed, "i");
+MODULE_PARM(io_speed, "i");
+
 /*====================================================================*/
 
 static socket_state_t dead_socket = {
@@ -994,11 +1018,23 @@ static int get_status(client_handle_t handle, status_t *status)
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
     s = SOCKET(handle);
-    c = CONFIG(handle);
     s->ss_entry(s->sock, SS_GetStatus, &val);
     status->CardState = status->SocketState = 0;
     status->CardState |= (val & SS_DETECT) ? CS_EVENT_CARD_DETECT : 0;
-    if ((c->state & CONFIG_LOCKED) && (c->IntType & INT_MEMORY_AND_IO)) {
+    if (s->state & SOCKET_SUSPEND)
+	status->CardState |= CS_EVENT_PM_SUSPEND;
+    if (!(s->state & SOCKET_PRESENT))
+	return CS_NO_CARD;
+
+    /* Get info from the PRR, if necessary */
+    if (handle->Function == BIND_FN_ALL) {
+	if ((s->functions > 0) && (status->Function > s->functions))
+	    return CS_BAD_ARGS;
+	c = (s->config != NULL) ? &s->config[status->Function] : NULL;
+    } else
+	c = CONFIG(handle);
+    if ((c != NULL) && (c->state & CONFIG_LOCKED) &&
+	(c->IntType & INT_MEMORY_AND_IO)) {
 	u_char reg;
 	if (c->Present & PRESENT_PIN_REPLACE) {
 	    read_cis_mem(s, 1, c->ConfigBase+CISREG_PRR, 1, &reg);
@@ -1016,19 +1052,16 @@ static int get_status(client_handle_t handle, status_t *status)
 	    status->CardState |=
 		(reg & ESR_REQ_ATTN) ? CS_EVENT_REQUEST_ATTENTION : 0;
 	}
+	return CS_SUCCESS;
     }
-    else {
-	status->CardState |=
-	    (val & SS_WRPROT) ? CS_EVENT_WRITE_PROTECT : 0;
-	status->CardState |=
-	    (val & SS_BATDEAD) ? CS_EVENT_BATTERY_DEAD : 0;
-	status->CardState |=
-	    (val & SS_BATWARN) ? CS_EVENT_BATTERY_LOW : 0;
-	status->CardState |=
-	    (val & SS_READY) ? CS_EVENT_READY_CHANGE : 0;
-    }
-    if (s->state & SOCKET_SUSPEND)
-	status->CardState |= CS_EVENT_PM_SUSPEND;
+    status->CardState |=
+	(val & SS_WRPROT) ? CS_EVENT_WRITE_PROTECT : 0;
+    status->CardState |=
+	(val & SS_BATDEAD) ? CS_EVENT_BATTERY_DEAD : 0;
+    status->CardState |=
+	(val & SS_BATWARN) ? CS_EVENT_BATTERY_LOW : 0;
+    status->CardState |=
+	(val & SS_READY) ? CS_EVENT_READY_CHANGE : 0;
     return CS_SUCCESS;
 } /* get_status */
 
@@ -1303,11 +1336,7 @@ static int release_irq(client_handle_t handle, irq_req_t *req)
     }
     
     if (req->Attributes & IRQ_HANDLE_PRESENT)
-#ifdef SA_SHIRQ
-	free_irq(req->AssignedIRQ, handle->dev_info);
-#else
-	free_irq(req->AssignedIRQ);
-#endif
+	FREE_IRQ(req->AssignedIRQ, handle->dev_info);
     
     undo_irq(req);
     
@@ -1481,10 +1510,10 @@ static int request_io(client_handle_t handle, io_req_t *req)
     if (!(s->state & SOCKET_PRESENT))
 	return CS_NO_CARD;
     c = CONFIG(handle);
-    if (c->state & CONFIG_IO_REQ)
-	return CS_IN_USE;
     if (c->state & CONFIG_LOCKED)
 	return CS_CONFIGURATION_LOCKED;
+    if (c->state & CONFIG_IO_REQ)
+	return CS_IN_USE;
     if (req->Attributes1 & (IO_SHARED | IO_FORCE_ALIAS_ACCESS))
 	return CS_BAD_ATTRIBUTE;
     if ((req->NumPorts2 > 0) &&
@@ -1533,10 +1562,10 @@ static int cs_request_irq(client_handle_t handle, irq_req_t *req)
     if (!(s->state & SOCKET_PRESENT))
 	return CS_NO_CARD;
     c = CONFIG(handle);
-    if (c->state & CONFIG_IRQ_REQ)
-	return CS_IN_USE;
     if (c->state & CONFIG_LOCKED)
 	return CS_CONFIGURATION_LOCKED;
+    if (c->state & CONFIG_IRQ_REQ)
+	return CS_IN_USE;
     
     ret = CS_BAD_ARGS;
     if (s->irq.AssignedIRQ != 0) {
@@ -1572,15 +1601,10 @@ static int cs_request_irq(client_handle_t handle, irq_req_t *req)
     if (ret != 0) return ret;
 
     if ((req->Attributes & IRQ_HANDLE_PRESENT) &&
-#ifdef SA_SHIRQ
-	request_irq(irq, req->Handler,
-		    ((s->functions > 1) ||
-		     (req->Attributes & IRQ_TYPE_DYNAMIC_SHARING))
-		    ? SA_SHIRQ : 0,
+	REQUEST_IRQ(irq, req->Handler,
+		    ((req->Attributes & IRQ_TYPE_DYNAMIC_SHARING) || 
+		     (s->functions > 1)) ? SA_SHIRQ : 0,
 		    (char *)handle->dev_info, handle->dev_info))
-#else
-	request_irq(irq, req->Handler, 0, (char *)handle->dev_info))
-#endif
 	return CS_IN_USE;
 
     c->irq.Attributes = req->Attributes;
