@@ -1,5 +1,5 @@
 /*
- *  cs_internal.h 1.8 1996/08/06 03:11:28 (David Hinds)
+ *  cs_internal.h 1.10 1996/11/26 07:20:07 (David Hinds)
  */
 
 #ifndef _LINUX_CS_INTERNAL_H
@@ -111,7 +111,7 @@ typedef struct socket_info_t {
     u_short		lock_count;
     client_handle_t	clients;
     client_handle_t	reset_handle;
-    struct timer_list	setup, shutdown, unreset;
+    struct timer_list	setup, shutdown;
     u_long		unreset_timeout;
     pcmcia_mem_map	cis_mem;
     config_t		*config;
@@ -166,6 +166,35 @@ typedef struct socket_info_t {
 #define EVENT(h, e, p) \
     ((h)->event_handler((e), (p), &(h)->event_callback_args))
 
+/* Stuff in cistpl.c */
+void read_cis_mem(socket_info_t *s, int attr,
+		  long addr, long len, u_char *ptr);
+void write_cis_mem(socket_info_t *s, int attr,
+		   long addr, long len, u_char *ptr);
+int setup_cis_mem(socket_info_t *s);
+int verify_cis_cache(socket_info_t *s);
+void preload_cis_cache(socket_info_t *s);
+int get_first_tuple(client_handle_t handle, tuple_t *tuple);
+int get_next_tuple(client_handle_t handle, tuple_t *tuple);
+int get_tuple_data(client_handle_t handle, tuple_t *tuple);
+int parse_tuple(client_handle_t handle, tuple_t *tuple, cisparse_t *parse);
+int validate_cis(client_handle_t handle, cisinfo_t *info);
+int read_tuple(client_handle_t handle, cisdata_t code, void *parse);
+
+/* In bulkmem.c */
+void retry_erase_list(struct erase_busy_t *list, u_long cause);
+int get_first_region(client_handle_t handle, region_info_t *rgn);
+int get_next_region(client_handle_t handle, region_info_t *rgn);
+int register_mtd(client_handle_t handle, mtd_reg_t *reg);
+int register_erase_queue(client_handle_t *handle, eraseq_hdr_t *header);
+int deregister_erase_queue(eraseq_handle_t eraseq);
+int check_erase_queue(eraseq_handle_t eraseq);
+int open_memory(client_handle_t *handle, open_mem_t *open);
+int close_memory(memory_handle_t handle);
+int read_memory(memory_handle_t handle, mem_op_t *req, caddr_t buf);
+int write_memory(memory_handle_t handle, mem_op_t *req, caddr_t buf);
+int copy_memory(memory_handle_t handle, copy_op_t *req);
+
 #define MAX_SOCK 8
 extern socket_t sockets;
 extern socket_info_t *socket_table[MAX_SOCK];
