@@ -20,6 +20,10 @@
 #include <linux/timer.h>
 #include <linux/ioctl.h>
 
+#if (LINUX_VERSION_CODE >= VERSION(2,1,23))
+#include <linux/poll.h>
+#endif
+
 #include <asm/segment.h>
 
 #include <pcmcia/version.h>
@@ -31,8 +35,9 @@
 
 #ifdef PCMCIA_DEBUG
 int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static const char *version =
-"ds.c 1.63 1996/12/21 21:34:41 (David Hinds)";
+"ds.c 1.67 1997/03/16 20:04:07 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -45,8 +50,12 @@ static FS_SIZE_T ds_read(struct inode *inode, struct file *file,
 			 char *buf, U_FS_SIZE_T count);
 static FS_SIZE_T ds_write(struct inode *inode, struct file *file,
 			  CONST char *buf, U_FS_SIZE_T count);
+#if (LINUX_VERSION_CODE < VERSION(2,1,23))
 static int ds_select(struct inode *inode, struct file *file,
 		     int sel_type, select_table *wait);
+#else
+static unsigned int ds_poll(struct file *file, poll_table *poll);
+#endif
 static int ds_ioctl(struct inode * inode, struct file * file,
 		    u_int cmd, u_long arg);
 static int ds_open(struct inode * inode, struct file * file);
@@ -109,7 +118,11 @@ static struct file_operations ds_fops = {
     ds_read,		/* read */
     ds_write,		/* write */
     NULL,		/* readdir */
+#if (LINUX_VERSION_CODE < VERSION(2,1,23))
     ds_select,		/* select */
+#else
+    ds_poll,		/* poll */
+#endif
     ds_ioctl,		/* ioctl */
     NULL,		/* mmap */
     ds_open,		/* open */
@@ -117,6 +130,8 @@ static struct file_operations ds_fops = {
     NULL		/* fsync */
 };
 
+#if (LINUX_VERSION_CODE <= VERSION(2,1,17))
+
 #undef CONFIG_MODVERSIONS
 static struct symbol_table ds_symtab = {
 #include <linux/symtab_begin.h>
@@ -131,6 +146,16 @@ static struct symbol_table ds_symtab = {
 #include <linux/symtab_end.h>
 };
 
+#else
+
+#define register_symtab(n)
+EXPORT_SYMBOL(register_pcmcia_driver);
+EXPORT_SYMBOL(unregister_pcmcia_driver);
+EXPORT_SYMBOL(bind_request);
+EXPORT_SYMBOL(bind_mtd);
+
+#endif
+
 /*====================================================================*/
 
 static void cs_error(int func, int ret)
@@ -626,6 +651,8 @@ static FS_SIZE_T ds_write(struct inode *inode, struct file *file,
 
 /*====================================================================*/
 
+#if (LINUX_VERSION_CODE < VERSION(2,1,23))
+
 static int ds_select(struct inode *inode, struct file *file,
 		     int sel_type, select_table *wait)
 {
@@ -652,6 +679,33 @@ static int ds_select(struct inode *inode, struct file *file,
     return 0;
 } /* ds_select */
 
+#else
+
+static unsigned int ds_poll(struct file *file, poll_table *wait)
+{
+    socket_t i = MINOR(file->f_inode->i_rdev);
+    socket_info_t *s;
+    user_info_t *user;
+
+#ifdef PCMCIA_DEBUG
+    if (pc_debug > 1)
+	printk(KERN_DEBUG "ds_poll(socket %d)\n", i);
+#endif
+    
+    if ((i >= sockets) || (sockets == 0))
+	return POLLERR;
+    s = &socket_table[i];
+    user = file->private_data;
+    if (CHECK_USER(user))
+	return POLLERR;
+    poll_wait(&s->queue, wait);
+    if (!queue_empty(user))
+	return POLLIN | POLLRDNORM;
+    return 0;
+} /* ds_poll */
+
+#endif
+
 /*====================================================================*/
 
 static int ds_ioctl(struct inode * inode, struct file * file,
