@@ -33,8 +33,9 @@
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"fixed_cs.c 1.11 1996/12/21 19:30:47 (David Hinds)";
+"fixed_cs.c 1.12 1997/03/16 20:08:38 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -44,6 +45,8 @@ static char *version =
 /* Bit map of interrupts to choose from */
 static u_long irq_mask = 0xdeb8;
 
+MODULE_PARM(irq_mask, "i");
+
 /*====================================================================*/
 
 static const char ide_major[4] = {
