@@ -51,6 +51,7 @@
 */
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 #endif
 
 /*
@@ -71,6 +72,9 @@ static u_long irq_mask = 0xdeb8;
 /* 0:4KB*2 TX buffer   else:8KB*2 TX buffer */
 static int sram_config = 0;
 
+MODULE_PARM(irq_mask, "i");
+MODULE_PARM(sram_config, "i");
+
 /*====================================================================*/
 /* 
    driver version infomation 
@@ -102,7 +106,7 @@ static int fjn_start_xmit(struct sk_buff *skb, struct device *dev);
 static void fjn_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs);
 static void fjn_rx(struct device *dev);
 static void fjn_reset(struct device *dev);
-static struct enet_statistics *fjn_get_stats(struct device *dev);
+static struct net_device_stats *fjn_get_stats(struct device *dev);
 
 #ifdef NEW_MULTICAST
 static void set_multicast_list(struct device *dev);
@@ -123,7 +127,7 @@ typedef enum { MBH10302, MBH10304, TDK, CONTEC } cardtype_t;
 */
 typedef struct local_info_t {
     dev_node_t node;
-    struct enet_statistics stats;
+    struct net_device_stats stats;
     long open_time;
     uint tx_started:1;
     uint tx_queue;
@@ -804,14 +808,13 @@ static int fjn_start_xmit(struct sk_buff *skb, struct device *dev)
 	sti();
     }
 
-    /* If some higher layer thinks we've missed an tx-done interrupt
-	   we are passed NULL. Caution: dev_tint() handles the cli()/sti()
-	   itself. */
+#if (LINUX_VERSION_CODE < VERSION(2,1,25))
     if (skb == NULL) {
 	dev_tint(dev);
 	return 0;
     }
-
+#endif
+    
     /* Block a timer-based transmit from overlapping.  This could better be
        done with atomic_swap(1, dev->tbusy), but set_bit() works as well. */
     if (set_bit(0, (void*)&dev->tbusy) != 0)
@@ -830,6 +833,9 @@ static int fjn_start_xmit(struct sk_buff *skb, struct device *dev)
 	if (fmvj18x_debug > 4)
 	    printk(KERN_DEBUG "%s: Transmitting a packet of length %lu.\n",
 		   dev->name, (unsigned long)skb->len);
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+	lp->stats.tx_bytes += skb->len;
+#endif
 
 	/* Disable both interrupts. */
 	outw(0x0000, ioaddr + TX_INTR);
@@ -1026,6 +1032,9 @@ static void fjn_rx(struct device *dev)
 
 	    netif_rx(skb);
 	    lp->stats.rx_packets++;
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+	    lp->stats.rx_bytes += skb->len;
+#endif
 	}
 	if (--boguscount <= 0)
 	    break;
@@ -1142,7 +1151,7 @@ static int fjn_close(struct device *dev)
 
 /*====================================================================*/
 
-static struct enet_statistics *fjn_get_stats(struct device *dev)
+static struct net_device_stats *fjn_get_stats(struct device *dev)
 {
     local_info_t *lp = (local_info_t *)dev->priv;
 
