@@ -58,8 +58,9 @@ static int major_dev = 0;
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"ftl_cs.c 1.25 1996/12/21 19:30:48 (David Hinds)";
+"ftl_cs.c 1.27 1997/03/16 20:08:38 (David Hinds)";
 #endif
 
 static int shuffle_freq = 50;
@@ -1076,7 +1077,11 @@ static void ftl_close(struct inode *inode, struct file *file)
 
     /* Flush all writes */
     fsync_dev(inode->i_rdev);
-
+#if (LINUX_VERSION_CODE >= VERSION(1,3,0))
+    invalidate_inodes(inode->i_rdev);
+#endif
+    invalidate_buffers(inode->i_rdev);
+    
     link = dev_table[DEVICE_NR(minor)];
     dev = (ftl_dev_t *)link->priv;
     part = &dev->minor[REGION_NR(minor)];
@@ -1336,28 +1341,31 @@ static int ftl_ioctl(struct inode *inode, struct file *file,
     dev_link_t *link;
     ftl_dev_t *dev;
     partition_t *partition;
-    int ret;
+    int ret = 0;
 
     link = dev_table[DEVICE_NR(minor)];
-    
-    if (!DEV_OK(link))
-	return -ENODEV;
+    if (!DEV_OK(link)) return -ENODEV;
     dev = (ftl_dev_t *)link->priv;
     partition = &dev->minor[REGION_NR(minor)];
+
     switch (cmd) {
     case BLKGETSIZE:
-	if (!(partition->state & FTL_FORMATTED))
-	    ret = -EIO;
-	else {
-	    ret = verify_area(VERIFY_WRITE, (long *)arg, sizeof(long));
-	    if (ret) break;
-	    put_user(partition->header.FormattedSize/SECTOR_SIZE,
-		     (long *)arg);
-	}
+	if (!(partition->state & FTL_FORMATTED)) return -EIO;
+	ret = verify_area(VERIFY_WRITE, (long *)arg, sizeof(long));
+	if (ret) return ret;
+	put_user(partition->header.FormattedSize/SECTOR_SIZE,
+		 (long *)arg);
+	break;
+    case BLKFLSBUF:
+	if (!suser()) return -EACCES;
+	if (!(inode->i_rdev)) return -EINVAL;
+	fsync_dev(inode->i_rdev);
+	invalidate_buffers(inode->i_rdev);
 	break;
     default:
 	ret = -EINVAL;
     }
+
     return ret;
 } /* ftl_ioctl */
 
