@@ -49,16 +49,19 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static const char *version =
-"i82365.c 1.104 1996/12/21 19:30:49 (David Hinds)";
+"i82365.c 1.108 1997/03/21 16:57:48 (David Hinds)";
 #endif
 
 /*====================================================================*/
 
 /* Parameters that can be set with 'insmod' */
 
-/* Default base address for i82365sl chip */
+/* Default base address for i82365sl and other ISA chips */
 static int i365_base = 0x3e0;
 
+/* Default base address for CardBus controllers */
+static int cardbus_base = 0x3e0;
+
 /* Specify a socket number to ignore */
 static int ignore = -1;
 
@@ -100,6 +103,28 @@ static int cmd_time = 6;
 static int recov_time = 0;
 static int wakeup = 0;
 
+MODULE_PARM(i365_base, "i");
+MODULE_PARM(cardbus_base, "i");
+MODULE_PARM(ignore, "i");
+MODULE_PARM(extra_sockets, "i");
+MODULE_PARM(do_scan, "i");
+MODULE_PARM(irq_mask, "i");
+MODULE_PARM(cs_irq, "i");
+MODULE_PARM(poll_interval, "i");
+MODULE_PARM(cycle_time, "i");
+MODULE_PARM(has_dma, "i");
+MODULE_PARM(has_led, "i");
+MODULE_PARM(has_ring, "i");
+MODULE_PARM(async_clock, "i");
+#ifdef CONFIG_PCI
+MODULE_PARM(fast_pci, "i");
+#endif
+MODULE_PARM(freq_bypass, "i");
+MODULE_PARM(setup_time, "i");
+MODULE_PARM(cmd_time, "i");
+MODULE_PARM(recov_time, "i");
+MODULE_PARM(wakeup, "i");
+
 /*====================================================================*/
 
 static struct timer_list poll_timer;
@@ -111,6 +136,7 @@ static int i365_service(u_int sock, u_int cmd, void *arg);
 typedef struct socket_info_t {
     int		type;
     socket_cap_t cap;
+    u_short	ioaddr;
     u_short	psock;
     u_short	cs_irq;
     void	(*handler)(void *info, u_long events);
@@ -133,11 +159,12 @@ typedef struct socket_info_t {
 #define IS_RL5C466	0x61
 #define IS_SMC34C90	0x70
 #define IS_TI113X	0x80
+#define IS_OM82C092G	0x90
 
 /* Where we keep track of our sockets... */
 static int sockets = 0;
 static socket_info_t socket_table[8] = {
-    { 0, { 0 }, 0, 0, NULL, NULL }, /* ... */
+    { 0, { 0 }, 0, 0, 0, NULL, NULL }, /* ... */
 };
 
 /* Socket capabilities */
@@ -149,8 +176,8 @@ static socket_cap_t i365_cap = {
 
 static u_char i365_get(u_short sock, u_short reg)
 {
-    u_short port = I365_PORT(sock);
-    u_char val = I365_REG(sock, reg);
+    u_short port = socket_table[sock].ioaddr;
+    u_char val = I365_REG(socket_table[sock].psock, reg);
     cli();
     outb_p(val, port); val = inb_p(port+1);
     sti();
@@ -159,8 +186,8 @@ static u_char i365_get(u_short sock, u_short reg)
 
 static void i365_set(u_short sock, u_short reg, u_char data)
 {
-    u_short port = I365_PORT(sock);
-    u_char val = I365_REG(sock, reg);
+    u_short port = socket_table[sock].ioaddr;
+    u_char val = I365_REG(socket_table[sock].psock, reg);
     cli();
     outb_p(val, port); outb_p(data, port+1);
     sti();
@@ -180,6 +207,16 @@ static void i365_bclr(u_short sock, u_short reg, u_char mask)
     i365_set(sock, reg, d);
 }
 
+static void i365_bflip(u_short sock, u_short reg, u_char mask, int b)
+{
+    u_char d = i365_get(sock, reg);
+    if (b)
+	d |= mask;
+    else
+	d &= ~mask;
+    i365_set(sock, reg, d);
+}
+
 static u_short i365_get_pair(u_short sock, u_short reg)
 {
     u_short a, b;
@@ -217,12 +254,12 @@ static void irq_count IRQ(int irq, void *dev, struct pt_regs *regs)
 
 static u_long try_irq(u_short psock, int irq)
 {
-    if (REQUEST_IRQ(irq, irq_count, 0, "irq scan") != 0)
+    if (REQUEST_IRQ(irq, irq_count, 0, "irq scan", NULL) != 0)
 	return -1;
     irq_hits = 0;
     busy_loop(HZ/50);
     if (irq_hits) {
-	FREE_IRQ(irq);
+	FREE_IRQ(irq, NULL);
 	return -1;
     }
 
@@ -232,7 +269,7 @@ static u_long try_irq(u_short psock, int irq)
     i365_bset(psock, I365_GENCTL, I365_CTL_SW_IRQ);
 
     busy_loop(HZ/50);
-    FREE_IRQ(irq);
+    FREE_IRQ(irq, NULL);
 
     /* Turn off interrupts */
     i365_set(psock, I365_CSCINT, 0);
@@ -283,12 +320,23 @@ static int to_ns(int cycles)
 
 /*====================================================================*/
 
-static int identify(int sock)
+static int identify(u_short port, u_short sock)
 {
     u_char val;
     int type = -1;
+
+    /* Use the next free entry in the socket table */
+    socket_table[sockets].ioaddr = port;
+    socket_table[sockets].psock = sock;
     
-    if ((val = i365_get(sock, I365_IDENT)) & 0x70)
+    /* Wake up a sleepy Cirrus controller */
+    if (wakeup) {
+	i365_bclr(sockets, PD67_MISC_CTL_2, PD67_MC2_SUSPEND);
+	/* Pause at least 50 ms */
+	busy_loop(HZ/20);
+    }
+    
+    if ((val = i365_get(sockets, I365_IDENT)) & 0x70)
 	return -1;
     switch (val) {
     case 0x82:
@@ -302,25 +350,25 @@ static int identify(int sock)
     }
     
     /* Check for Vadem VG-468 chips */
-    outb_p(0x0e, I365_PORT(sock));
-    outb_p(0x37, I365_PORT(sock));
-    i365_bset(sock, VG468_MISC, VG468_MISC_VADEMREV);
-    val = i365_get(sock, I365_IDENT);
+    outb_p(0x0e, port);
+    outb_p(0x37, port);
+    i365_bset(sockets, VG468_MISC, VG468_MISC_VADEMREV);
+    val = i365_get(sockets, I365_IDENT);
     if (val & I365_IDENT_VADEM) {
-	i365_bclr(sock, VG468_MISC, VG468_MISC_VADEMREV);
+	i365_bclr(sockets, VG468_MISC, VG468_MISC_VADEMREV);
 	type = ((val & 7) >= 4) ? IS_VG469 : IS_VG468;
     }
 
     /* Check for Ricoh chips */
-    val = i365_get(sock, RF5C_CHIP_ID);
+    val = i365_get(sockets, RF5C_CHIP_ID);
     if ((val == RF5C_CHIP_RF5C296) || (val == RF5C_CHIP_RF5C396))
 	type = IS_RF5Cx96;
     
     /* Check for Cirrus CL-PD67xx chips */
-    i365_set(sock, PD67_CHIP_INFO, 0);
-    val = i365_get(sock, PD67_CHIP_INFO);
+    i365_set(sockets, PD67_CHIP_INFO, 0);
+    val = i365_get(sockets, PD67_CHIP_INFO);
     if ((val & PD67_INFO_CHIP_ID) == PD67_INFO_CHIP_ID) {
-	val = i365_get(sock, PD67_CHIP_INFO);
+	val = i365_get(sockets, PD67_CHIP_INFO);
 	if ((val & PD67_INFO_CHIP_ID) == 0)
 	    type = (val & PD67_INFO_SLOTS) ? IS_PD672X : IS_PD6710;
     }
@@ -329,19 +377,20 @@ static int identify(int sock)
 
 /*====================================================================*/
 
-static void add_socket(int type, int psock)
+static void add_socket(u_short port, int psock, int type)
 {
+    socket_table[sockets].ioaddr = port;
     socket_table[sockets].psock = psock;
     socket_table[sockets].type = type;
     sockets++;
 }
 
-static void add_pcic(int type, int ns)
+static void add_pcic(u_short port, int ns, int type)
 {
-    int i, j, irq;
-    u_short base;
+    int i, j, irq, base;
     u_long mask, scan;
-    
+
+    request_region(port, 2, "i82365");
     if (sockets == ns) printk("\n" KERN_INFO "  ");
     switch (type) {
     case IS_VLSI:
@@ -378,12 +427,14 @@ static void add_pcic(int type, int ns)
 	printk("SMC 34C90 CardBus"); break;
     case IS_TI113X:
 	printk("TI 113X CardBus"); break;
+    case IS_OM82C092G:
+	printk("Omega Micro 82C092G PCI"); break;
     }
-    base = sockets-ns;
-    printk(" at %#x", I365_PORT(socket_table[base].psock));
+    printk(" at %#x", port);
     printk(", %d socket%s\n", ns, ((ns > 1) ? "s" : ""));
 
     /* Build interrupt mask; scan if possible */
+    base = sockets-ns;
     printk(KERN_INFO "  irq mask (");
     mask = i365_cap.irq_mask & irq_mask;
     switch (type) {
@@ -420,12 +471,12 @@ static void add_pcic(int type, int ns)
 	if (cs_irq == 0) {
 	    for (i = 15; i > 0; i--)
 		if ((i != 12) && (mask & (1 << i)) &&
-		    (REQUEST_IRQ(i, i365_interrupt, 0, "i82365") == 0))
+		    (REQUEST_IRQ(i, i365_interrupt, 0, "i82365", NULL) == 0))
 		    break;
 	    irq = i;
 	}
 	else {
-	    if (REQUEST_IRQ(cs_irq, i365_interrupt, 0, "i82365") == 0)
+	    if (REQUEST_IRQ(cs_irq, i365_interrupt, 0, "i82365", NULL) == 0)
 		irq = cs_irq;
 	}
 	if (irq == 0) poll_interval = HZ;
@@ -459,17 +510,21 @@ static void add_pcic(int type, int ns)
     
 ======================================================================*/
 
-static int is_active(int s)
+static int is_active(u_short port, u_short sock)
 {
     u_char stat;
     u_short start, stop;
     
-    stat = i365_get(s, I365_STATUS);
-    start = i365_get_pair(s, I365_IO(0)+I365_W_START);
-    stop = i365_get_pair(s, I365_IO(0)+I365_W_STOP);
+    /* Use the next free entry in the socket table */
+    socket_table[sockets].ioaddr = port;
+    socket_table[sockets].psock = sock;
+    
+    stat = i365_get(sockets, I365_STATUS);
+    start = i365_get_pair(sockets, I365_IO(0)+I365_W_START);
+    stop = i365_get_pair(sockets, I365_IO(0)+I365_W_STOP);
     if ((stat & I365_CS_DETECT) && (stat & I365_CS_POWERON) &&
-	(i365_get(s, I365_INTCTL) & I365_PC_IOCARD) &&
-	(i365_get(s, I365_ADDRWIN) & I365_ENA_IO(0)) &&
+	(i365_get(sockets, I365_INTCTL) & I365_PC_IOCARD) &&
+	(i365_get(sockets, I365_ADDRWIN) & I365_ENA_IO(0)) &&
 	(check_region(start, stop-start+1) != 0) &&
 	((start & 0xfeef) != 0x02e8))
 	return 1;
@@ -481,6 +536,15 @@ static int is_active(int s)
 
 #ifdef CONFIG_PCI
 
+#ifndef PCI_VENDOR_ID_OMEGA
+#define PCI_VENDOR_ID_OMEGA 0x119b
+#define PCI_DEVICE_ID_OMEGA_82C092G 0x1221
+#endif
+
+#ifndef PCI_FUNC
+#define PCI_FUNC(devfn) ((devfn)&7)
+#endif
+
 typedef struct {
     u_short	vendor;
     u_short	device;
@@ -499,66 +563,88 @@ static pci_id_t pci_id[] = {
 #define CMD_DFLT (PCI_COMMAND_IO|PCI_COMMAND_MEMORY| \
 		  PCI_COMMAND_MASTER|PCI_COMMAND_WAIT)
 
-static void pci_setup(int type, u_char bus, u_char fn)
+static void pci_setup(int type, u_char bus, u_char devfn, u_short addr)
 {
     u_short s;
 
-    pcibios_write_config_word(bus, fn, PCI_COMMAND, CMD_DFLT);
+    pcibios_write_config_word(bus, devfn, PCI_COMMAND, CMD_DFLT);
     switch (type) {
     case IS_TI113X:
-	pcibios_read_config_word(bus, fn, TI113X_DCR, &s);
+	pcibios_read_config_word(bus, devfn, TI113X_DCR, &s);
 	s = (s & ~TI113X_DCR_IMODE_MASK) | TI113X_DCR_IMODE_ISA;
-	pcibios_write_config_word(bus, fn, TI113X_DCR, s);
+	pcibios_write_config_word(bus, devfn, TI113X_DCR, s);
 	break;
     case IS_RL5C466:
-	s = RL5C466_16CTL_LEVEL_1;
-	pcibios_write_config_word(bus, fn, RL5C466_16BIT_CTL, s);
+	s = RL5C466_16CTL_LEVEL_1 | RL5C466_16CTL_LEVEL_2 |
+	    RL5C466_16CTL_IO_TIMING | RL5C466_16CTL_MEM_TIMING;
+	pcibios_write_config_word(bus, devfn, RL5C466_16BIT_CTL, s);
 	break;
     case IS_PD6832:
-	pcibios_read_config_word(bus, fn, CB_BRIDGE_CONTROL, &s);
+	pcibios_read_config_word(bus, devfn, CB_BRIDGE_CONTROL, &s);
 	s |= PD6832_BCR_MGMT_IRQ_ENA;
-	pcibios_write_config_word(bus, fn, CB_BRIDGE_CONTROL, s);
+	pcibios_write_config_word(bus, devfn, CB_BRIDGE_CONTROL, s);
 	break;
     }
     /* Enable ISA interrupt steering */
-    pcibios_read_config_word(bus, fn, CB_BRIDGE_CONTROL, &s);
+    pcibios_read_config_word(bus, devfn, CB_BRIDGE_CONTROL, &s);
     s |= CB_BCR_ISA_IRQ;
-    pcibios_write_config_word(bus, fn, CB_BRIDGE_CONTROL, s);
+    pcibios_write_config_word(bus, devfn, CB_BRIDGE_CONTROL, s);
     /* Enable 16-bit legacy mode */
-    pcibios_write_config_dword(bus, fn, CB_LEGACY_MODE_BASE,
-			       i365_base | 0x01);
+    pcibios_write_config_dword(bus, devfn, CB_LEGACY_MODE_BASE,
+			       addr | 0x01);
 }
 
 static void pci_probe(void)
 {
     int i, index;
-    u_char bus, fn;
+    u_char bus, devfn;
     u_int addr;
     
-    /* Check for Cirrus PD6729 PCI controller */
-    if (pcibios_find_device(PCI_VENDOR_ID_CIRRUS,
-			    PCI_DEVICE_ID_CIRRUS_6729,
-			    0, &bus, &fn) == 0) {
-	pcibios_read_config_dword(bus, fn, PCI_BASE_ADDRESS_0, &addr);
-	i365_base = addr & ~0x1;
-	pcibios_write_config_word(bus, fn, PCI_COMMAND, CMD_DFLT);
-	add_socket(IS_PD6729, 0);
-	add_socket(IS_PD6729, 1);
-	add_pcic(IS_PD6729, 2);
+    /* Check for Cirrus PD6729 PCI controller(s) */
+    for (i = 0; ; i++) {
+	if (pcibios_find_device(PCI_VENDOR_ID_CIRRUS,
+				PCI_DEVICE_ID_CIRRUS_6729,
+				i, &bus, &devfn) != 0)
+	    break;
+	pcibios_read_config_dword(bus, devfn, PCI_BASE_ADDRESS_0, &addr);
+	addr &= ~0x1;
+	pcibios_write_config_word(bus, devfn, PCI_COMMAND, CMD_DFLT);
+	add_socket(addr, 0, IS_PD6729);
+	add_socket(addr, 1, IS_PD6729);
+	add_pcic(addr, 2, IS_PD6729);
+    }
+    
+    /* Check for Omega Micro 82C092G controller(s) */
+    for (i = 0; ; i++) {
+	if (pcibios_find_device(PCI_VENDOR_ID_OMEGA,
+				PCI_DEVICE_ID_OMEGA_82C092G,
+				i, &bus, &devfn) != 0)
+	    break;
+	pcibios_read_config_dword(bus, devfn, PCI_BASE_ADDRESS_0, &addr);
+        addr &= ~0x1;
+	pcibios_write_config_word(bus, devfn, PCI_COMMAND, CMD_DFLT);
+	add_socket(addr, 0, IS_OM82C092G);
+	add_socket(addr, 1, IS_OM82C092G);
+	add_pcic(addr, 2, IS_OM82C092G);
     }
 
     /* Check for CardBus controllers */
+    addr = cardbus_base;
     for (i = 0; i < PCI_COUNT; i++) {
-	for (index = 0; index < 2; index++) {
+	for (index = 0; ; index++) {
 	    if (pcibios_find_device(pci_id[i].vendor,
 				    pci_id[i].device,
-				    index, &bus, &fn) != 0)
+				    index, &bus, &devfn) != 0)
 		break;
-	    pci_setup(pci_id[i].type, bus, fn);
-	    add_socket(pci_id[i].type, index);
+	    if (PCI_FUNC(devfn) == 0) {
+		pci_setup(pci_id[i].type, bus, devfn, addr);
+		pci_setup(pci_id[i].type, bus, devfn+1, addr);
+		add_socket(addr, 0, pci_id[i].type);
+		add_socket(addr, 1, pci_id[i].type);
+		add_pcic(addr, 2, pci_id[i].type);
+		addr += 4;
+	    }
 	}
-	if (index != 0)
-	    add_pcic(pci_id[i].type, index);
     }
 
 }
@@ -568,59 +654,58 @@ static void pci_probe(void)
 
 static void isa_probe(void)
 {
-    int i, j, k, p, ns;
+    int i, j, sock, k, ns;
     int id, type;
+    u_short port;
     
-    if (check_region(i365_base, 8) != 0) {
-	printk("port conflict at %#x\n", i365_base);
+    if (check_region(i365_base, 2) != 0) {
+	if (sockets == 0)
+	    printk("port conflict at %#x\n", i365_base);
 	return;
     }
 
-    /* Wake up a sleepy Cirrus controller */
-    if (wakeup) {
-	i365_bclr(0, PD67_MISC_CTL_2, PD67_MC2_SUSPEND);
-	/* Pause at least 50 ms */
-	busy_loop(HZ/20);
-    }
-    
-    type = identify(0);
-    if ((type == IS_I82365DF) && (identify(1) != type)) {
+    type = identify(i365_base, 0);
+    if ((type == IS_I82365DF) && (identify(i365_base, 1) != type)) {
 	for (i = 0; i < 4; i++) {
 	    if (i == ignore) continue;
-	    j = ((i & 1) << 3) + (i << 1);
-	    if ((identify(j) == IS_I82365DF) && !is_active(j))
-		add_socket(IS_VLSI, j);
+	    port = i365_base + ((i & 1) << 1) + (i>>1);
+	    sock = (i<<1) & 3;
+	    if ((identify(port, sock) == IS_I82365DF) &&
+		!is_active(port, sock)) {
+		add_socket(port, sock, IS_VLSI);
+		add_pcic(port, 1, IS_VLSI);
+	    }
 	}
-	add_pcic(IS_VLSI, sockets);
     } else {
 	for (i = 0; i < (extra_sockets ? 8 : 4); i += 2) {
-	    id = identify(i);
+	    port = i365_base + (i>>2);
+	    sock = (i & 3);
+	    id = identify(port, sock);
 	    if (id < 0) continue;
 
 	    ns = 0;
 	    for (j = 0; j < 2; j++) {
 		/* Does the socket exist? */
-		if ((ignore == i+j) || (identify(i+j) < 0) ||
-		    is_active(i+j))
+		if ((ignore == i+j) || (identify(port, sock+j) < 0) ||
+		    is_active(port, sock+j))
 		    continue;
 		/* Check for bad socket decode */
 		i365_set(i+j, I365_MEM(0)+I365_W_OFF, i+j);
 		for (k = 0; k < sockets; k++) {
-		    p = socket_table[k].psock;
-		    if (i365_get(p, I365_MEM(0)+I365_W_OFF) != p) {
-			i365_set(p, I365_MEM(0)+I365_W_OFF, p);
+		    if (i365_get(k, I365_MEM(0)+I365_W_OFF) != k) {
+			i365_set(k, I365_MEM(0)+I365_W_OFF, k);
 			break;
 		    }
 		}
 		if (i365_get(i+j, I365_MEM(0)+I365_W_OFF) != i+j)
 		    break;
 		if (k == sockets) {
-		    add_socket(type, i+j);
+		    add_socket(port, sock+j, type);
 		    ns++;
 		}
 	    }
 	    if (ns != 0)
-		add_pcic(type, ns);
+		add_pcic(port, ns, type);
 	}
     }
 }
@@ -643,16 +728,13 @@ static int i365_init(void)
 	pci_probe();
 #endif
 
-    if (sockets == 0)
-	isa_probe();
+    isa_probe();
 	
     if (sockets == 0) {
 	printk("not found.\n");
 	return -ENODEV;
     }
     
-    request_region(i365_base, 8, "i82365");
-
     /* Set up polling */
     if (poll_interval != 0) {
 	poll_timer.function = (void (*)(u_long))i365_interrupt;
@@ -663,11 +745,11 @@ static int i365_init(void)
     if (register_ss_entry(sockets, &i365_service) != 0) {
 	int this_irq, last_irq = 0;
 	printk(KERN_NOTICE "i82365: register_ss_entry() failed\n");
-	release_region(i365_base, 8);
 	for (i = 0; i < sockets; i++) {
 	    this_irq = socket_table[i].cs_irq;
 	    if (this_irq && (this_irq != last_irq))
-		FREE_IRQ(last_irq = this_irq);
+		FREE_IRQ(last_irq = this_irq, NULL);
+	    release_region(socket_table[i].ioaddr, 2);
 	}
 	return -ENODEV;
     }
@@ -694,9 +776,9 @@ static void i365_finish(void)
 	i365_set(socket_table[i].psock, I365_CSCINT, 0);
 	this_irq = socket_table[i].cs_irq;
 	if (this_irq && (this_irq != last_irq))
-	    FREE_IRQ(last_irq = this_irq);
+	    FREE_IRQ(last_irq = this_irq, NULL);
+	release_region(socket_table[i].ioaddr, 2);
     }
-    release_region(i365_base, 8);
 } /* i365_finish */
 
 /*====================================================================*/
@@ -704,7 +786,6 @@ static void i365_finish(void)
 static void i365_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
 {
     int i, csc;
-    u_short psock;
     u_long events;
     
 #ifdef PCMCIA_DEBUG
@@ -721,11 +802,10 @@ static void i365_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
     
     for (i = 0; i < sockets; i++) {
 	if (socket_table[i].handler == NULL) continue;
-	psock = socket_table[i].psock;
-	csc = i365_get(psock, I365_CSC);
+	csc = i365_get(i, I365_CSC);
 	if (csc == 0) continue;
 	events = (csc & I365_CSC_DETECT) ? SS_DETECT : 0;
-	if (i365_get(psock, I365_INTCTL) & I365_PC_IOCARD)
+	if (i365_get(i, I365_INTCTL) & I365_PC_IOCARD)
 	    events |= (csc & I365_CSC_STSCHG) ? SS_STSCHG : 0;
 	else {
 	    events |= (csc & I365_CSC_BVD1) ? SS_BATDEAD : 0;
@@ -752,31 +832,30 @@ static void i365_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
 
 /*====================================================================*/
 
-static int i365_register_callback(u_short lsock, ss_callback_t *call)
+static int i365_register_callback(u_short sock, ss_callback_t *call)
 {
     if (call == NULL) {
-	socket_table[lsock].handler = NULL;
+	socket_table[sock].handler = NULL;
 	MOD_DEC_USE_COUNT;
     }
     else {
 	MOD_INC_USE_COUNT;
-	socket_table[lsock].handler = call->handler;
-	socket_table[lsock].info = call->info;
+	socket_table[sock].handler = call->handler;
+	socket_table[sock].info = call->info;
     }
     return 0;
 } /* i365_register_callback */
 
 /*====================================================================*/
 
-static int i365_get_status(u_short lsock, u_int *value)
+static int i365_get_status(u_short sock, u_int *value)
 {
-    u_short psock = socket_table[lsock].psock;
     u_char status;
     
-    status = i365_get(psock, I365_STATUS);
+    status = i365_get(sock, I365_STATUS);
     *value = ((status & I365_CS_DETECT) == I365_CS_DETECT)
 	? SS_DETECT : 0;
-    if (i365_get(psock, I365_INTCTL) & I365_PC_IOCARD)
+    if (i365_get(sock, I365_INTCTL) & I365_PC_IOCARD)
 	*value |= (status & I365_CS_STSCHG) ? 0 : SS_STSCHG;
     else {
 	*value |= (status & I365_CS_BVD1) ? 0 : SS_BATDEAD;
@@ -788,36 +867,35 @@ static int i365_get_status(u_short lsock, u_int *value)
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
 	printk(KERN_DEBUG "i82365: GetStatus(%d) = %#2.2x\n",
-	       lsock, *value);
+	       sock, *value);
 #endif
     return 0;
 } /* i365_get_status */
 
 /*====================================================================*/
 
-static int i365_inquire_socket(u_short lsock, socket_cap_t *cap)
+static int i365_inquire_socket(u_short sock, socket_cap_t *cap)
 {
-    *cap = socket_table[lsock].cap;
+    *cap = socket_table[sock].cap;
     return 0;
 } /* i365_inquire_socket */
 
 /*====================================================================*/
 
-static int i365_get_socket(u_short lsock, socket_state_t *state)
+static int i365_get_socket(u_short sock, socket_state_t *state)
 {
-    u_short psock = socket_table[lsock].psock;
     u_char reg, vcc, vpp;
     
-    reg = i365_get(psock, I365_POWER);
+    reg = i365_get(sock, I365_POWER);
     state->flags = (reg & I365_PWR_AUTO) ? SS_PWR_AUTO : 0;
     state->flags |= (reg & I365_PWR_OUT) ? SS_OUTPUT_ENA : 0;
     vcc = reg & I365_VCC_MASK; vpp = reg & I365_VPP_MASK;
     state->Vcc = state->Vpp = 0;
-    switch (socket_table[lsock].type) {
+    switch (socket_table[sock].type) {
     case IS_PD672X:
     case IS_PD6710:
     case IS_PD6729:
-	if (i365_get(psock, PD67_MISC_CTL_1) & PD67_MC1_VCC_3V) {
+	if (i365_get(sock, PD67_MISC_CTL_1) & PD67_MC1_VCC_3V) {
 	    if (reg & I365_VCC_5V) state->Vcc = 33;
 	    if (vpp == I365_VPP_5V) state->Vpp = 33;
 	    if (vpp == I365_VPP_12V) state->Vpp = 120;
@@ -829,7 +907,7 @@ static int i365_get_socket(u_short lsock, socket_state_t *state)
 	}
 	break;
     case IS_VG469:
-	if (i365_get(psock, VG469_VSELECT) & VG469_VSEL_VCC) {
+	if (i365_get(sock, VG469_VSELECT) & VG469_VSEL_VCC) {
 	    if (reg & I365_VCC_5V) state->Vcc = 33;
 	    if (vpp == I365_VPP_5V) state->Vpp = 33;
 	    if (vpp == I365_VPP_12V) state->Vpp = 120;
@@ -854,31 +932,31 @@ static int i365_get_socket(u_short lsock, socket_state_t *state)
 	}
 	break;
     }
-    reg = i365_get(psock, I365_INTCTL);
+    reg = i365_get(sock, I365_INTCTL);
     state->flags |= (reg & I365_PC_IOCARD) ? SS_IOCARD : 0;
     state->flags |= (reg & I365_PC_RESET) ? 0 : SS_RESET;
 
     /* IO card, RESET flags, IO interrupt */
-    reg = i365_get(psock, I365_INTCTL);
+    reg = i365_get(sock, I365_INTCTL);
     state->flags |= (reg & I365_PC_RESET) ? 0 : SS_RESET;
     if (reg & I365_PC_IOCARD) state->flags |= SS_IOCARD;
     state->io_irq = reg & I365_IRQ_MASK;
     
     /* DMA mode, speaker control */
-    switch (socket_table[lsock].type) {
+    switch (socket_table[sock].type) {
     case IS_PD672X:
     case IS_PD6710:
     case IS_PD6729:
-	if (i365_get(psock, PD67_MISC_CTL_1) & PD67_MC1_SPKR_ENA)
+	if (i365_get(sock, PD67_MISC_CTL_1) & PD67_MC1_SPKR_ENA)
 	    state->flags |= SS_SPKR_ENA;
-	if (i365_get(psock, PD67_MISC_CTL_2) & PD67_MC2_DMA_MODE)
+	if (i365_get(sock, PD67_MISC_CTL_2) & PD67_MC2_DMA_MODE)
 	    state->flags |= SS_DMA_MODE;
 	break;
     default:
     }
     
     /* Card status change mask */
-    reg = i365_get(psock, I365_CSCINT);
+    reg = i365_get(sock, I365_CSCINT);
     state->csc_mask = (reg & I365_CSC_DETECT) ? SS_DETECT : 0;
     if (state->flags & SS_IOCARD)
 	state->csc_mask |= (reg & I365_CSC_STSCHG) ? SS_STSCHG : 0;
@@ -891,7 +969,7 @@ static int i365_get_socket(u_short lsock, socket_state_t *state)
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
 	printk(KERN_DEBUG "i82365: GetSocket(%d) = flags %#3.3lx, "
-	       "Vcc %d, Vpp %d, io_irq %d, csc_mask %#2.2lx\n", lsock,
+	       "Vcc %d, Vpp %d, io_irq %d, csc_mask %#2.2lx\n", sock,
 	       state->flags, state->Vcc, state->Vpp, state->io_irq,
 	       state->csc_mask);
 #endif
@@ -900,38 +978,37 @@ static int i365_get_socket(u_short lsock, socket_state_t *state)
 
 /*====================================================================*/
 
-static int i365_set_socket(u_short lsock, socket_state_t *state)
+static int i365_set_socket(u_short sock, socket_state_t *state)
 {
-    int type = socket_table[lsock].type;
-    u_short psock = socket_table[lsock].psock;
+    int type = socket_table[sock].type;
     u_char reg;
     
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
 	printk(KERN_DEBUG "i82365: SetSocket(%d, flags %#3.3lx, "
 	       "Vcc %d, Vpp %d, io_irq %d, csc_mask %#2.2lx)\n",
-	       lsock, state->flags, state->Vcc, state->Vpp,
+	       sock, state->flags, state->Vcc, state->Vpp,
 	       state->io_irq, state->csc_mask);
 #endif
 
     /* Did something put the controller to sleep? */
     if ((type == IS_PD6710) || (type == IS_PD672X) ||
 	(type == IS_PD6729)) {
-	if (i365_get(0, PD67_MISC_CTL_2) & PD67_MC2_SUSPEND) {
-	    i365_bclr(0, PD67_MISC_CTL_2, PD67_MC2_SUSPEND);
+	if (i365_get(sock, PD67_MISC_CTL_2) & PD67_MC2_SUSPEND) {
+	    i365_bclr(sock, PD67_MISC_CTL_2, PD67_MC2_SUSPEND);
 	    /* Pause at least 50 ms */
 	    busy_loop(HZ/20);
 	}
     }
     
     /* IO card, RESET flags, IO interrupt */
-    reg = i365_get(psock, I365_INTCTL);
+    reg = i365_get(sock, I365_INTCTL);
     reg &= ~(I365_PC_IOCARD | I365_PC_RESET | I365_IRQ_MASK);
     /* Note that the reset signal is inverted */
     reg |= (state->flags & SS_RESET) ? 0 : I365_PC_RESET;
     reg |= (state->flags & SS_IOCARD) ? I365_PC_IOCARD : 0;
     reg |= state->io_irq;
-    i365_set(psock, I365_INTCTL, reg);
+    i365_set(sock, I365_INTCTL, reg);
 
     reg = I365_PWR_NORESET;
     if (state->flags & SS_PWR_AUTO) reg |= I365_PWR_AUTO;
@@ -954,16 +1031,16 @@ static int i365_set_socket(u_short lsock, socket_state_t *state)
 	    if (state->Vcc == 33) {
 		reg |= I365_VCC_5V;
 		if (type == IS_VG469)
-		    i365_bset(psock, VG469_VSELECT, VG469_VSEL_VCC);
+		    i365_bset(sock, VG469_VSELECT, VG469_VSEL_VCC);
 		else /* Cirrus */
-		    i365_bset(psock, PD67_MISC_CTL_1, PD67_MC1_VCC_3V);
+		    i365_bset(sock, PD67_MISC_CTL_1, PD67_MC1_VCC_3V);
 	    }
 	    else if (state->Vcc == 50) {
 		reg |= I365_VCC_5V;
 		if (type == IS_VG469)
-		    i365_bclr(psock, VG469_VSELECT, VG469_VSEL_VCC);
+		    i365_bclr(sock, VG469_VSELECT, VG469_VSEL_VCC);
 		else /* Cirrus */
-		    i365_bclr(psock, PD67_MISC_CTL_1, PD67_MC1_VCC_3V);
+		    i365_bclr(sock, PD67_MISC_CTL_1, PD67_MC1_VCC_3V);
 	    }
 	    else return -EINVAL;
 	}
@@ -1000,68 +1077,57 @@ static int i365_set_socket(u_short lsock, socket_state_t *state)
 	
     }
     
-    if (reg != i365_get(psock, I365_POWER))
-	i365_set(psock, I365_POWER, reg);
+    if (reg != i365_get(sock, I365_POWER))
+	i365_set(sock, I365_POWER, reg);
 
     /* Chipset-specific functions */
     switch (type) {
     case IS_VG468:
     case IS_VG469:
-	if (async_clock)
-	    i365_bset(psock, VG468_CTL, VG468_CTL_ASYNC);
-	else
-	    i365_bclr(psock, VG468_CTL, VG468_CTL_ASYNC);
+	i365_bflip(sock, VG468_CTL, VG468_CTL_ASYNC, async_clock);
 	break;
 #ifdef CONFIG_PCI
     case IS_PD6729:
-	if (fast_pci)
-	    i365_bset(psock, PD67_MISC_CTL_2, PD67_MC2_FAST_PCI);
+	i365_bflip(sock, PD67_MISC_CTL_2, PD67_MC2_FAST_PCI, fast_pci);
 	/* Fall through... */
 #endif
     case IS_PD6710:
     case IS_PD672X:
 	/* Special pin functions */
-	if (has_led)
-	    i365_bset(psock, PD67_MISC_CTL_2, PD67_MC2_LED_ENA);
-	if (has_ring)
-	    i365_bset(psock, PD67_MISC_CTL_2, PD67_MC2_IRQ15_RI);
-	else
-	    i365_bclr(psock, PD67_MISC_CTL_2, PD67_MC2_IRQ15_RI);
+	i365_bclr(sock, PD67_MISC_CTL_2, PD67_MC2_DYNAMIC_MODE);
+	i365_bflip(sock, PD67_MISC_CTL_2, PD67_MC2_LED_ENA, has_led);
+	i365_bflip(sock, PD67_MISC_CTL_2, PD67_MC2_IRQ15_RI, has_ring);
 	/* Speaker control */
-	if (state->flags & SS_SPKR_ENA)
-	    i365_bset(psock, PD67_MISC_CTL_1, PD67_MC1_SPKR_ENA);
-	else
-	    i365_bclr(psock, PD67_MISC_CTL_1, PD67_MC1_SPKR_ENA);
+	i365_bflip(sock, PD67_MISC_CTL_1, PD67_MC1_SPKR_ENA,
+		   state->flags & SS_SPKR_ENA);
 	/* DMA control */
-	i365_set(psock, PD67_EXT_INDEX, PD67_DMA_CTL);
-	reg = i365_get(psock, PD67_EXT_DATA) & ~PD67_DMA_MODE;
+	i365_set(sock, PD67_EXT_INDEX, PD67_DMA_CTL);
+	reg = i365_get(sock, PD67_EXT_DATA) & ~PD67_DMA_MODE;
 	if (state->flags & SS_DMA_MODE) {
 	    reg |= PD67_DMA_DREQ_BVD2;
-	    i365_set(psock, PD67_EXT_DATA, reg);
-	    i365_bset(psock, PD67_MISC_CTL_2, PD67_MC2_DMA_MODE);
+	    i365_set(sock, PD67_EXT_DATA, reg);
+	    i365_bset(sock, PD67_MISC_CTL_2, PD67_MC2_DMA_MODE);
 	}
 	else {
-	    i365_bclr(psock, PD67_MISC_CTL_2, PD67_MC2_DMA_MODE);
-	    i365_set(psock, PD67_EXT_DATA, reg);
+	    i365_bclr(sock, PD67_MISC_CTL_2, PD67_MC2_DMA_MODE);
+	    i365_set(sock, PD67_EXT_DATA, reg);
 	}
 	/* Timing parameters */
-	if (freq_bypass)
-	    i365_bset(psock, PD67_MISC_CTL_2, PD67_MC2_FREQ_BYPASS);
-	else
-	    i365_bclr(psock, PD67_MISC_CTL_2, PD67_MC2_FREQ_BYPASS);
-	i365_set(psock, PD67_TIME_SETUP(0), setup_time);
-	i365_set(psock, PD67_TIME_SETUP(1), setup_time);
-	i365_set(psock, PD67_TIME_CMD(0), cmd_time);
-	i365_set(psock, PD67_TIME_CMD(1), cmd_time + 9);
-	i365_set(psock, PD67_TIME_RECOV(0), recov_time);
-	i365_set(psock, PD67_TIME_RECOV(1), recov_time);
+	i365_bflip(sock, PD67_MISC_CTL_2, PD67_MC2_FREQ_BYPASS,
+		   freq_bypass);
+	i365_set(sock, PD67_TIME_SETUP(0), setup_time);
+	i365_set(sock, PD67_TIME_SETUP(1), setup_time);
+	i365_set(sock, PD67_TIME_CMD(0), cmd_time);
+	i365_set(sock, PD67_TIME_CMD(1), cmd_time + 9);
+	i365_set(sock, PD67_TIME_RECOV(0), recov_time);
+	i365_set(sock, PD67_TIME_RECOV(1), recov_time);
 	break;
     default:
 	break;
     }
     
     /* Card status change interrupt mask */
-    reg = (socket_table[lsock].cs_irq << 4);
+    reg = (socket_table[sock].cs_irq << 4);
     if (state->csc_mask & SS_DETECT) reg |= I365_CSC_DETECT;
     if (state->flags & SS_IOCARD) {
 	if (state->csc_mask & SS_STSCHG) reg |= I365_CSC_STSCHG;
@@ -1071,25 +1137,24 @@ static int i365_set_socket(u_short lsock, socket_state_t *state)
 	if (state->csc_mask & SS_BATWARN) reg |= I365_CSC_BVD2;
 	if (state->csc_mask & SS_READY) reg |= I365_CSC_READY;
     }
-    i365_set(psock, I365_CSCINT, reg);
-    i365_get(psock, I365_CSC);
+    i365_set(sock, I365_CSCINT, reg);
+    i365_get(sock, I365_CSC);
     
     return 0;
 } /* i365_set_socket */
 
 /*====================================================================*/
 
-static int i365_get_io_map(u_short lsock, struct pcmcia_io_map *io)
+static int i365_get_io_map(u_short sock, struct pcmcia_io_map *io)
 {
-    u_short psock = socket_table[lsock].psock;
     u_char map, ioctl, addr;
     
     map = io->map;
     if (map > 1) return -EINVAL;
-    io->start = i365_get_pair(psock, I365_IO(map)+I365_W_START);
-    io->stop = i365_get_pair(psock, I365_IO(map)+I365_W_STOP);
-    ioctl = i365_get(psock, I365_IOCTL);
-    addr = i365_get(psock, I365_ADDRWIN);
+    io->start = i365_get_pair(sock, I365_IO(map)+I365_W_START);
+    io->stop = i365_get_pair(sock, I365_IO(map)+I365_W_STOP);
+    ioctl = i365_get(sock, I365_IOCTL);
+    addr = i365_get(sock, I365_ADDRWIN);
     io->speed = to_ns(ioctl & I365_IOCTL_WAIT(map)) ? 1 : 0;
     io->flags  = (addr & I365_ENA_IO(map)) ? MAP_ACTIVE : 0;
     io->flags |= (ioctl & I365_IOCTL_0WS(map)) ? MAP_0WS : 0;
@@ -1098,7 +1163,7 @@ static int i365_get_io_map(u_short lsock, struct pcmcia_io_map *io)
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
 	printk(KERN_DEBUG "i82365: GetIOMap(%d, %d) = %#2.2x, %d ns, "
-	       "%#4.4x-%#4.4x\n", lsock, map, io->flags,
+	       "%#4.4x-%#4.4x\n", sock, map, io->flags,
 	       io->speed, io->start, io->stop);
 #endif
     return 0;
@@ -1106,63 +1171,61 @@ static int i365_get_io_map(u_short lsock, struct pcmcia_io_map *io)
 
 /*====================================================================*/
 
-static int i365_set_io_map(u_short lsock, struct pcmcia_io_map *io)
+static int i365_set_io_map(u_short sock, struct pcmcia_io_map *io)
 {
-    u_short psock = socket_table[lsock].psock;
     u_char map, ioctl;
     
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
 	printk(KERN_DEBUG "i82365: SetIOMap(%d, %d, %#2.2x, %d ns, "
-	       "%#4.4x-%#4.4x)\n", lsock, io->map, io->flags,
+	       "%#4.4x-%#4.4x)\n", sock, io->map, io->flags,
 	       io->speed, io->start, io->stop);
 #endif
     map = io->map;
     if ((map > 1) || (io->start > 0xffff) || (io->stop > 0xffff) ||
 	(io->stop < io->start)) return -EINVAL;
     /* Turn off the window before changing anything */
-    if (i365_get(psock, I365_ADDRWIN) & I365_ENA_IO(map))
-	i365_bclr(psock, I365_ADDRWIN, I365_ENA_IO(map));
-    i365_set_pair(psock, I365_IO(map)+I365_W_START, io->start);
-    i365_set_pair(psock, I365_IO(map)+I365_W_STOP, io->stop);
-    ioctl = i365_get(psock, I365_IOCTL) & ~I365_IOCTL_MASK(map);
+    if (i365_get(sock, I365_ADDRWIN) & I365_ENA_IO(map))
+	i365_bclr(sock, I365_ADDRWIN, I365_ENA_IO(map));
+    i365_set_pair(sock, I365_IO(map)+I365_W_START, io->start);
+    i365_set_pair(sock, I365_IO(map)+I365_W_STOP, io->stop);
+    ioctl = i365_get(sock, I365_IOCTL) & ~I365_IOCTL_MASK(map);
     if (io->speed) ioctl |= I365_IOCTL_WAIT(map);
     if (io->flags & MAP_0WS) ioctl |= I365_IOCTL_0WS(map);
     if (io->flags & MAP_16BIT) ioctl |= I365_IOCTL_16BIT(map);
     if (io->flags & MAP_AUTOSZ) ioctl |= I365_IOCTL_IOCS16(map);
-    i365_set(psock, I365_IOCTL, ioctl);
+    i365_set(sock, I365_IOCTL, ioctl);
     /* Turn on the window if necessary */
     if (io->flags & MAP_ACTIVE)
-	i365_bset(psock, I365_ADDRWIN, I365_ENA_IO(map));
+	i365_bset(sock, I365_ADDRWIN, I365_ENA_IO(map));
     return 0;
 } /* i365_set_io_map */
 
 /*====================================================================*/
 
-static int i365_get_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
+static int i365_get_mem_map(u_short sock, struct pcmcia_mem_map *mem)
 {
-    u_short psock = socket_table[lsock].psock;
     u_short base, i;
     u_char map, addr;
     
     map = mem->map;
     if (map > 4) return -EINVAL;
-    addr = i365_get(psock, I365_ADDRWIN);
+    addr = i365_get(sock, I365_ADDRWIN);
     mem->flags = (addr & I365_ENA_MEM(map)) ? MAP_ACTIVE : 0;
     base = I365_MEM(map);
     
-    i = i365_get_pair(psock, base+I365_W_START);
+    i = i365_get_pair(sock, base+I365_W_START);
     mem->flags |= (i & I365_MEM_16BIT) ? MAP_16BIT : 0;
     mem->flags |= (i & I365_MEM_0WS) ? MAP_0WS : 0;
     mem->sys_start = ((long)(i & 0x0fff) << 12);
     
-    i = i365_get_pair(psock, base+I365_W_STOP);
+    i = i365_get_pair(sock, base+I365_W_STOP);
     mem->speed  = (i & I365_MEM_WS0) ? 1 : 0;
     mem->speed += (i & I365_MEM_WS1) ? 2 : 0;
     mem->speed = to_ns(mem->speed);
     mem->sys_stop = ((long)(i & 0x0fff) << 12) + 0x0fff;
     
-    i = i365_get_pair(psock, base+I365_W_OFF);
+    i = i365_get_pair(sock, base+I365_W_OFF);
     mem->flags |= (i & I365_MEM_WRPROT) ? MAP_WRPROT : 0;
     mem->flags |= (i & I365_MEM_REG) ? MAP_ATTRIB : 0;
     mem->card_start = ((long)(i & 0x3fff) << 12) + mem->sys_start;
@@ -1170,7 +1233,7 @@ static int i365_get_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
 	printk(KERN_DEBUG "i82365: GetMemMap(%d, %d) = %#2.2x, %d ns, "
-	       "%#5.5lx-%#5.5lx, %#5.5lx\n", lsock, mem->map,
+	       "%#5.5lx-%#5.5lx, %#5.5lx\n", sock, mem->map,
 	       mem->flags, mem->speed, mem->sys_start, mem->sys_stop,
 	       mem->card_start);
 #endif
@@ -1179,16 +1242,15 @@ static int i365_get_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
 
 /*====================================================================*/
   
-static int i365_set_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
+static int i365_set_mem_map(u_short sock, struct pcmcia_mem_map *mem)
 {
-    u_short psock = socket_table[lsock].psock;
     u_short base, i;
     u_char map;
     
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
 	printk(KERN_DEBUG "i82365: SetMemMap(%d, %d, %#2.2x, %d ns, "
-	       "%#5.5lx-%#5.5lx, %#5.5lx)\n", lsock, mem->map,
+	       "%#5.5lx-%#5.5lx, %#5.5lx)\n", sock, mem->map,
 	       mem->flags, mem->speed, mem->sys_start, mem->sys_stop,
 	       mem->card_start);
 #endif
@@ -1198,14 +1260,14 @@ static int i365_set_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
 	(mem->sys_start > mem->sys_stop) || (mem->speed > 1000))
 	return -EINVAL;
     /* Turn off the window before changing anything */
-    if (i365_get(psock, I365_ADDRWIN) & I365_ENA_MEM(map))
-	i365_bclr(psock, I365_ADDRWIN, I365_ENA_MEM(map));
+    if (i365_get(sock, I365_ADDRWIN) & I365_ENA_MEM(map))
+	i365_bclr(sock, I365_ADDRWIN, I365_ENA_MEM(map));
     base = I365_MEM(map);
     
     i = mem->sys_start >> 12;
     if (mem->flags & MAP_16BIT) i |= I365_MEM_16BIT;
     if (mem->flags & MAP_0WS) i |= I365_MEM_0WS;
-    i365_set_pair(psock, base+I365_W_START, i);
+    i365_set_pair(sock, base+I365_W_START, i);
     
     i = mem->sys_stop >> 12;
     switch (to_cycles(mem->speed)) {
@@ -1214,16 +1276,16 @@ static int i365_set_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
     case 2:	i |= I365_MEM_WS1; break;
     default:	i |= I365_MEM_WS1 | I365_MEM_WS0; break;
     }
-    i365_set_pair(psock, base+I365_W_STOP, i);
+    i365_set_pair(sock, base+I365_W_STOP, i);
     
     i = ((mem->card_start - mem->sys_start) >> 12) & 0x3fff;
     if (mem->flags & MAP_WRPROT) i |= I365_MEM_WRPROT;
     if (mem->flags & MAP_ATTRIB) i |= I365_MEM_REG;
-    i365_set_pair(psock, base+I365_W_OFF, i);
+    i365_set_pair(sock, base+I365_W_OFF, i);
     
     /* Turn on the window if necessary */
     if (mem->flags & MAP_ACTIVE)
-	i365_bset(psock, I365_ADDRWIN, I365_ENA_MEM(map));
+	i365_bset(sock, I365_ADDRWIN, I365_ENA_MEM(map));
     return 0;
 } /* i365_set_mem_map */
 
@@ -1245,18 +1307,18 @@ static subfn_t service_table[] = {
 
 #define NFUNC (sizeof(service_table)/sizeof(subfn_t))
 
-static int i365_service(u_int lsock, u_int cmd, void *arg)
+static int i365_service(u_int sock, u_int cmd, void *arg)
 {
     int err;
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1) 
 	printk(KERN_DEBUG "i365_ioctl(%d, %d, 0x%p)\n",
-	       lsock, cmd, arg);
+	       sock, cmd, arg);
 #endif
 
     if (cmd < NFUNC)
-	err = service_table[cmd](lsock, arg);
+	err = service_table[cmd](sock, arg);
     else
 	err = -EINVAL;
 
