@@ -39,6 +39,7 @@ extern struct device *irq2dev_map[16];
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
 "ibmtr_cs.c 1.10 1996/01/06 05:19:00 (Steve Kipisz)";
 #endif
@@ -65,6 +66,13 @@ static int ringspeed = 16;
 /* Ugh!  Let the user hardwire the hardware address for queer cards */
 static int hw_addr[6] = { 0, /* ... */ };
 
+MODULE_PARM(irq_mask, "i");
+MODULE_PARM(mmiobase, "i");
+MODULE_PARM(srambase, "i");
+MODULE_PARM(sramsize, "i");
+MODULE_PARM(ringspeed, "i");
+MODULE_PARM(hw_addr, "6i");
+
 /*====================================================================*/
 
 static void ibmtr_config(dev_link_t *link);
