@@ -41,21 +41,30 @@
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"iflash2+_mtd.c 1.29 1996/12/21 19:30:50 (David Hinds)";
+"iflash2+_mtd.c 1.30 1997/03/16 20:08:00 (David Hinds)";
 #endif
 
 /*====================================================================*/
 
+/* Parameters that can be set with 'insmod' */
+
 static int vpp_timeout_period	= HZ;		/* in ticks */
 static int vpp_settle		= HZ/10;	/* in ticks */
 static int write_timeout	= HZ/10;	/* in ticks */
 static int erase_timeout	= 100;		/* in ms */
 static int erase_limit		= 10*HZ;	/* in ticks */
+static int retry_limit		= 4;		/* write retries */
+static u_long max_tries       	= 4096;		/* status polling */
 
-static int retry_limit		= 4;	/* max retries */
-
-static u_long max_tries       	= 4096;
+MODULE_PARM(vpp_timeout_period, "i");
+MODULE_PARM(vpp_settle, "i");
+MODULE_PARM(write_timeout, "i");
+MODULE_PARM(erase_timeout, "i");
+MODULE_PARM(erase_limit, "i");
+MODULE_PARM(retry_limit, "i");
+MODULE_PARM(max_tries, "i");
 
 /*====================================================================*/
 
