@@ -54,8 +54,9 @@ static int major_dev = 0;
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"memory_cs.c 1.26 1996/12/21 19:30:52 (David Hinds)";
+"memory_cs.c 1.28 1997/03/16 20:10:53 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -108,6 +109,7 @@ static FS_SIZE_T memory_write(struct inode *inode, struct file *file,
 			      CONST char *buf, U_FS_SIZE_T count);
 static int memory_open(struct inode *inode, struct file *file);
 static void memory_close(struct inode *inode, struct file *file);
+static void memory_blk_close(struct inode *inode, struct file *file);
 
 static struct file_operations memory_chr_fops = {
     NULL,		/* lseek */
@@ -131,7 +133,7 @@ static struct file_operations memory_blk_fops = {
     memory_ioctl,	/* ioctl */
     NULL,		/* mmap */
     memory_open,	/* open */
-    memory_close,	/* release */
+    memory_blk_close,	/* release */
     block_fsync		/* fsync */
 };
 
@@ -471,7 +473,7 @@ static int memory_open(struct inode *inode, struct file *file)
 	open.Offset = minor_dev->region.CardOffset;
 	ret = CardServices(OpenMemory, &minor_dev->handle, &open);
 	if (ret != CS_SUCCESS)
-	    return -ENODEV;
+	    return -ENOMEM;
     }
     file->private_data = minor_dev;
     
@@ -504,11 +506,21 @@ static void memory_close(struct inode *inode, struct file *file)
 	CardServices(CloseMemory, minor_dev->handle);
 	minor_dev->handle = NULL;
     }
-    
+
     MOD_DEC_USE_COUNT;
 
 } /* memory_close */
 
+static void memory_blk_close(struct inode *inode, struct file *file)
+{
+    fsync_dev(inode->i_rdev);
+#if (LINUX_VERSION_CODE >= VERSION(1,3,0))
+    invalidate_inodes(inode->i_rdev);
+#endif
+    invalidate_buffers(inode->i_rdev);
+    memory_close(inode, file);
+}
+
 /*======================================================================
 
     Read for character-mode device
@@ -644,7 +656,7 @@ static int memory_ioctl(struct inode *inode, struct file *file,
     minor_dev_t *minor_dev;
     erase_info_t erase;
     u_int size;
-    int ret;
+    int ret = 0;
 
     link = dev_table[DEVICE_NR(minor)];
     
@@ -663,12 +675,17 @@ static int memory_ioctl(struct inode *inode, struct file *file,
 	if (ret) return ret;
     }
 
-    ret = 0;
     switch (cmd) {
     case BLKGETSIZE:
 	put_user(minor_dev->region.RegionSize/SECTOR_SIZE,
 		 (long *)arg);
 	break;
+    case BLKFLSBUF:
+	if (!suser()) return -EACCES;
+	if (!(inode->i_rdev)) return -EINVAL;
+	fsync_dev(inode->i_rdev);
+	invalidate_buffers(inode->i_rdev);
+	break;
     case MEMGETINFO:
 	copy_to_user((region_info_t *)arg, &minor_dev->region,
 		     sizeof(struct region_info_t));
@@ -681,6 +698,7 @@ static int memory_ioctl(struct inode *inode, struct file *file,
     default:
 	ret = -EINVAL;
     }
+    
     return ret;
 } /* memory_ioctl */
 
