@@ -367,7 +367,7 @@ typedef struct _mace_statistics {
 
 typedef struct _mace_private {
   dev_node_t node;
-  struct enet_statistics linux_stats; /* Linux statistics counters */
+  struct net_device_stats linux_stats; /* Linux statistics counters */
   mace_statistics mace_stats; /* MACE chip statistics counters */
 
   /* restore_multicast_list() state variables */
@@ -402,6 +402,7 @@ static char *if_names[]={
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 #endif
 
 /* #define NMCLAN_DEBUG 6 */
@@ -411,6 +412,7 @@ static int nmclan_debug = NMCLAN_DEBUG;
 static int nmclan_debug = 2;
 #endif
 
+MODULE_PARM(nmclan_debug, "i");
 
 /* ----------------------------------------------------------------------------
 Parameters
@@ -428,6 +430,8 @@ static int if_port=0; /* default=auto */
 /* Bit map of interrupts to choose from */
 static u_long irq_mask=0xdeb8; /* 3, 4, 5, 7, 9, 10, 11, 12, 14, 15 */
 
+MODULE_PARM(if_port, "i");
+MODULE_PARM(irq_mask, "i");
 
 /* ----------------------------------------------------------------------------
 Function Prototypes
@@ -444,7 +448,7 @@ static int mace_open(struct device *dev);
 static int mace_close(struct device *dev);
 static int mace_start_xmit(struct sk_buff *skb, struct device *dev);
 static void mace_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs);
-static struct enet_statistics *mace_get_stats(struct device *dev);
+static struct net_device_stats *mace_get_stats(struct device *dev);
 static int mace_rx(struct device *dev, unsigned char RxCnt);
 static void restore_multicast_list(struct device *dev);
 
@@ -1193,13 +1197,14 @@ static int mace_start_xmit(struct sk_buff *skb, struct device *dev)
     return 1;
 #endif /* #if TIMEOUT_TX */
 
+#if (LINUX_VERSION_CODE < VERSION(2,1,25))
   if (skb == NULL) {
     dev_tint(dev);
     return 0;
   }
-
   if (skb->len <= 0)
     return 0;
+#endif
  
 #ifdef NMCLAN_DEBUG
   if (nmclan_debug > 4) {
@@ -1230,6 +1235,9 @@ static int mace_start_xmit(struct sk_buff *skb, struct device *dev)
     */
     set_bit(TBUSY_PARTIAL_TX_FRAME, (void*)&dev->tbusy);
 
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+    lp->linux_stats.tx_bytes += skb->len;
+#endif
     lp->tx_free_frames--;
     set_bit(TBUSY_NO_FREE_TX_FRAMES, (void*)&dev->tbusy);
 
@@ -1501,6 +1509,9 @@ static int mace_rx(struct device *dev, unsigned char RxCnt)
 	netif_rx(skb); /* Send the packet to the upper (protocol) layers. */
 
 	lp->linux_stats.rx_packets++;
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+	lp->linux_stats.rx_bytes += skb->len;
+#endif
 	outb(0xFF, ioaddr + AM2150_RCV_NEXT); /* skip to next frame */
 	continue;
       } else {
@@ -1520,31 +1531,31 @@ static int mace_rx(struct device *dev, unsigned char RxCnt)
 /* ----------------------------------------------------------------------------
 pr_linux_stats
 ---------------------------------------------------------------------------- */
-static void pr_linux_stats(struct enet_statistics *pstats)
+static void pr_linux_stats(struct net_device_stats *pstats)
 {
   printk(KERN_DEBUG "pr_linux_stats\n");
-  printk(KERN_DEBUG " rx_packets=%-7d        tx_packets=%d\n",
-    pstats->rx_packets, pstats->tx_packets);
-  printk(KERN_DEBUG " rx_errors=%-7d         tx_errors=%d\n",
-    pstats->rx_errors, pstats->tx_errors);
-  printk(KERN_DEBUG " rx_dropped=%-7d        tx_dropped=%d\n",
-    pstats->rx_dropped, pstats->tx_dropped);
-  printk(KERN_DEBUG " multicast=%-7d         collisions=%d\n",
-    pstats->multicast, pstats->collisions);
+  printk(KERN_DEBUG " rx_packets=%-7ld        tx_packets=%ld\n",
+    (long)pstats->rx_packets, (long)pstats->tx_packets);
+  printk(KERN_DEBUG " rx_errors=%-7ld         tx_errors=%ld\n",
+    (long)pstats->rx_errors, (long)pstats->tx_errors);
+  printk(KERN_DEBUG " rx_dropped=%-7ld        tx_dropped=%ld\n",
+    (long)pstats->rx_dropped, (long)pstats->tx_dropped);
+  printk(KERN_DEBUG " multicast=%-7ld         collisions=%ld\n",
+    (long)pstats->multicast, (long)pstats->collisions);
 
-  printk(KERN_DEBUG " rx_length_errors=%-7d  rx_over_errors=%d\n",
-    pstats->rx_length_errors, pstats->rx_over_errors);
-  printk(KERN_DEBUG " rx_crc_errors=%-7d     rx_frame_errors=%d\n",
-    pstats->rx_crc_errors, pstats->rx_frame_errors);
-  printk(KERN_DEBUG " rx_fifo_errors=%-7d    rx_missed_errors=%d\n",
-    pstats->rx_fifo_errors, pstats->rx_missed_errors);
+  printk(KERN_DEBUG " rx_length_errors=%-7ld  rx_over_errors=%ld\n",
+    (long)pstats->rx_length_errors, (long)pstats->rx_over_errors);
+  printk(KERN_DEBUG " rx_crc_errors=%-7ld     rx_frame_errors=%ld\n",
+    (long)pstats->rx_crc_errors, (long)pstats->rx_frame_errors);
+  printk(KERN_DEBUG " rx_fifo_errors=%-7ld    rx_missed_errors=%ld\n",
+    (long)pstats->rx_fifo_errors, (long)pstats->rx_missed_errors);
 
-  printk(KERN_DEBUG " tx_aborted_errors=%-7d tx_carrier_errors=%d\n",
-    pstats->tx_aborted_errors, pstats->tx_carrier_errors);
-  printk(KERN_DEBUG " tx_fifo_errors=%-7d    tx_heartbeat_errors=%d\n",
-    pstats->tx_fifo_errors, pstats->tx_heartbeat_errors);
-  printk(KERN_DEBUG " tx_window_errors=%d\n",
-    pstats->tx_window_errors);
+  printk(KERN_DEBUG " tx_aborted_errors=%-7ld tx_carrier_errors=%ld\n",
+    (long)pstats->tx_aborted_errors, (long)pstats->tx_carrier_errors);
+  printk(KERN_DEBUG " tx_fifo_errors=%-7ld    tx_heartbeat_errors=%ld\n",
+    (long)pstats->tx_fifo_errors, (long)pstats->tx_heartbeat_errors);
+  printk(KERN_DEBUG " tx_window_errors=%ld\n",
+    (long)pstats->tx_window_errors);
 } /* pr_linux_stats */
 
 
@@ -1673,7 +1684,7 @@ static void update_stats(int ioaddr, struct device *dev)
 mace_get_stats
 	Gathers ethernet statistics from the MACE chip.
 ---------------------------------------------------------------------------- */
-static struct enet_statistics *mace_get_stats(struct device *dev)
+static struct net_device_stats *mace_get_stats(struct device *dev)
 {
   mace_private *lp = (mace_private *)dev->priv;
 
@@ -1687,13 +1698,16 @@ static struct enet_statistics *mace_get_stats(struct device *dev)
   }
 
   return &lp->linux_stats;
-} /* enet_statistics */
+} /* net_device_stats */
 
 
 /* ----------------------------------------------------------------------------
 updateCRC
 	Modified from Am79C90 data sheet.
 ---------------------------------------------------------------------------- */
+
+#if BROKEN_MULTICAST
+
 static void updateCRC(int *CRC, int bit)
 {
   int poly[]={
@@ -1879,6 +1893,57 @@ static void set_multicast_list(struct device *dev, int num_addrs, void *addrs)
 
 } /* set_multicast_list */
 
+#endif /* BROKEN_MULTICAST */
+
+static void restore_multicast_list(struct device *dev)
+{
+  mace_private *lp = (mace_private *)dev->priv;
+  int num_addrs = lp->multicast_num_addrs;
+  short ioaddr = dev->base_addr;
+
+  if (nmclan_debug > 3)
+    printk(KERN_DEBUG "%s: restoring Rx mode to %d addresses.\n",
+	   dev->name, num_addrs);
+  
+#ifdef NEW_MULTICAST
+  if (dev->flags & IFF_PROMISC) {
+#else
+  if (num_addrs < 0) {
+#endif
+    /* Promiscuous mode: receive all packets */
+    mace_write(ioaddr, MACE_UTR, MACE_UTR_LOOP_EXTERNAL);
+    mace_write(ioaddr, MACE_MACCC,
+      MACE_MACCC_PROM | MACE_MACCC_ENXMT | MACE_MACCC_ENRCV
+    );
+  } else {
+    /* Normal mode */
+    mace_write(ioaddr, MACE_UTR, MACE_UTR_LOOP_EXTERNAL);
+    mace_write(ioaddr, MACE_MACCC, MACE_MACCC_ENXMT | MACE_MACCC_ENRCV);
+  }
+} /* restore_multicast_list */
+
+#ifdef NEW_MULTICAST
+#define num_addrs dev->mc_count
+static void set_multicast_list(struct device *dev)
+#else
+static void set_multicast_list(struct device *dev, int num_addrs, void *addrs)
+#endif
+{
+  mace_private *lp = (mace_private *)dev->priv;
+
+  if (nmclan_debug > 1) {
+    static int old = 0;
+    if (num_addrs != old) {
+      old = num_addrs;
+      printk(KERN_INFO "%s: setting Rx mode to %d addresses.\n",
+	     dev->name, old);
+    }
+  }
+
+  lp->multicast_num_addrs = num_addrs;
+  restore_multicast_list(dev);
+
+} /* set_multicast_list */
 
 /* ----------------------------------------------------------------------------
 init_module
