@@ -50,8 +50,9 @@ static int major_dev = 0;
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"pcmem_cs.c 1.68 1996/12/21 19:30:54 (David Hinds)";
+"pcmem_cs.c 1.70 1997/03/16 20:08:18 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -64,6 +65,9 @@ static int word_width = 1;
 /* Speed of memory accesses, in ns */
 static int mem_speed = 0;
 
+MODULE_PARM(word_width, "i");
+MODULE_PARM(mem_speed, "i");
+
 /*====================================================================*/
 
 /* Maximum number of separate memory devices we'll allow */
@@ -103,8 +107,6 @@ static int pcmem_blocksizes[MINOR_NR(MAX_DEV)] =
     
 /*====================================================================*/
 
-static int pcmem_lseek(struct inode *inode, struct file *file,
-		       off_t offset, int orig);
 static int pcmem_ioctl(struct inode *inode, struct file *file,
 		       u_int cmd, u_long arg);
 static FS_SIZE_T pcmem_read(struct inode *inode, struct file *file,
@@ -113,9 +115,10 @@ static FS_SIZE_T pcmem_write(struct inode *inode, struct file *file,
 			     CONST char *buf, U_FS_SIZE_T count);
 static int pcmem_open(struct inode *inode, struct file *file);
 static void pcmem_close(struct inode *inode, struct file *file);
+static void pcmem_blk_close(struct inode *inode, struct file *file);
 
 static struct file_operations pcmem_chr_fops = {
-    pcmem_lseek,	/* lseek */
+    NULL,		/* lseek */
     pcmem_read,		/* read */
     pcmem_write,	/* write */
     NULL,		/* readdir */
@@ -136,7 +139,7 @@ static struct file_operations pcmem_blk_fops = {
     pcmem_ioctl,	/* ioctl */
     NULL,		/* mmap */
     pcmem_open,		/* open */
-    pcmem_close,	/* release */
+    pcmem_blk_close,	/* release */
     block_fsync		/* fsync */
 };
 
@@ -515,66 +518,20 @@ static void pcmem_close(struct inode *inode, struct file *file)
 
     link = dev_table[DEVICE_NR(inode->i_rdev)];
     if (link == NULL) return;
-    
+
     MOD_DEC_USE_COUNT;
 
 } /* pcmem_close */
 
-/*======================================================================
-
-    I'm not sure if this is actually necessary.  It might be nicer
-    when we actually determine the device size by reading the CIS.
-    
-======================================================================*/
-
-static int pcmem_lseek(struct inode *inode, struct file *file,
-		       off_t offset, int orig)
+static void pcmem_blk_close(struct inode *inode, struct file *file)
 {
-    dev_link_t *link;
-    pcmem_dev_t *dev;
-    
-#ifdef PCMCIA_DEBUG
-    int minor = MINOR(inode->i_rdev);
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_lseek(%d, %lu, %d)\n",
-	       minor, offset, orig);
+    fsync_dev(inode->i_rdev);
+#if (LINUX_VERSION_CODE >= VERSION(1,3,0))
+    invalidate_inodes(inode->i_rdev);
 #endif
-    /* Let's just make sure there really is a card here! */
-    link = dev_table[DEVICE_NR(inode->i_rdev)];
-    if (!DEV_OK(link))
-	return -ENODEV;
-
-    dev = (pcmem_dev_t *)link->priv;
-    
-    switch (orig) {
-    case 0:
-	if ((offset < 0) || (offset > HIGH_ADDR))
-	    return -EINVAL;
-        if (offset > dev->size)
-       	    return -ENOSPC;
-	file->f_pos = offset;
-	break;
-    case 1:
-	if ((file->f_pos+offset < 0) ||
-	    (file->f_pos+offset > HIGH_ADDR))
-	    return -EINVAL;
-	if (file->f_pos+offset > dev->size)
-	    return -ENOSPC;
-	file->f_pos += offset;
-	break;
-    case 2:
-	if (((int)dev->size+offset < 0) ||
-	    (dev->size+offset > HIGH_ADDR))
-	    return -EINVAL;
-        if (dev->size+offset > dev->size)
-       	    return -ENOSPC;
-	file->f_pos = dev->size+offset;
-	break;
-    default:
-	return -EINVAL;
-    }
-    return file->f_pos;
-} /* pcmem_lseek */
+    invalidate_buffers(inode->i_rdev);
+    pcmem_close(inode, file);
+}
 
 /*======================================================================
 
@@ -754,23 +711,25 @@ static int pcmem_ioctl(struct inode *inode, struct file *file,
 {
     dev_link_t *link;
     pcmem_dev_t *dev;
-    int ret;
+    int ret = 0;
     
     link = dev_table[DEVICE_NR(inode->i_rdev)];
-    
-    if (!DEV_OK(link))
-	return -ENODEV;
+    if (!DEV_OK(link)) return -ENODEV;
     dev = (pcmem_dev_t *)link->priv;
+
     switch (cmd) {
     case BLKGETSIZE:
-	if (dev->size == 0) {
-	    ret = -EINVAL;
-	    break;
-	}
+	if (dev->size == 0) return -EINVAL;
 	ret = verify_area(VERIFY_WRITE, (long *)arg, sizeof(long));
 	if (ret) break;
 	put_user(dev->size/SECTOR_SIZE, (long *)arg);
 	break;
+    case BLKFLSBUF:
+	if (!suser()) return -EACCES;
+	if (!(inode->i_rdev)) return -EINVAL;
+	fsync_dev(inode->i_rdev);
+	invalidate_buffers(inode->i_rdev);
+	break;
     default:
 	ret = -EINVAL;
     }
