@@ -68,8 +68,9 @@ static char *if_names[] = { "Auto", "10baseT", "10base2"};
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"pcnet_cs.c 1.44 1996/12/21 19:30:55 (David Hinds)";
+"pcnet_cs.c 1.48 1997/03/16 20:09:13 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -100,6 +101,15 @@ static int use_shmem = -1;
 /* Ugh!  Let the user hardwire the hardware address for queer cards */
 static int hw_addr[6] = { 0, /* ... */ };
 
+MODULE_PARM(irq_mask, "i");
+MODULE_PARM(if_port, "i");
+MODULE_PARM(use_big_buf, "i");
+MODULE_PARM(mem_speed, "i");
+MODULE_PARM(delay_output, "i");
+MODULE_PARM(delay_time, "i");
+MODULE_PARM(use_shmem, "i");
+MODULE_PARM(hw_addr, "6i");
+
 /*====================================================================*/
 
 static void pcnet_config(dev_link_t *link);
@@ -140,48 +150,55 @@ typedef struct hw_info_t {
 #define USE_SHMEM	0x04
 #define USE_BIG_BUF	0x08
 #define HAS_IBM_MISC	0x10
+#define IS_MULTIFN	0x20
 
 static hw_info_t hw_info[] = {
     { "Accton EN2212", 0x0ff0, 0x00, 0x00, 0xe8, DELAY_OUTPUT },
-    { "Allied Telesis LA-PCM", 0x0ff0, 0x00, 0x00, 0xf4, 0 },
+    { "Allied Telesis LA-PCM", 0x0ff0, 0x00, 0x00, 0xf4, USE_SHMEM },
     { "APEX MultiCard", 0x03f4, 0x00, 0x20, 0xe5, 0 },
+    { "ASANTE FriendlyNet", 0x4910, 0x00, 0x00, 0x94,
+      DELAY_OUTPUT | HAS_IBM_MISC },
     { "Danpex EN-6200P2", 0x0110, 0x00, 0x40, 0xc7, 0 },
     { "DataTrek NetCard", 0x0ff0, 0x00, 0x20, 0xe8, 0 },
+    { "Dayna CommuniCard E", 0x0110, 0x00, 0x80, 0x19, 0 },
     { "D-Link DE-650", 0x0040, 0x00, 0x80, 0xc8, 0 },
     { "EP-210 Ethernet", 0x0110, 0x00, 0x40, 0x33, 0 },
     { "Epson EEN10B", 0x0ff0, 0x00, 0x00, 0x48,
-          USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "ELECOM Laneed LD-CDWA", 0xb8, 0x08, 0x00, 0x42, 0 },
     { "Grey Cell GCS2220", 0x0000, 0x00, 0x47, 0x43, 0 },
-    { "IBM CCAE", 0x0ff0, 0x08, 0x00, 0x5a,
-	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "IBM CCAE", 0x0ff0, 0x00, 0x04, 0xac,
-          USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "IBM CCAE", 0x0ff0, 0x00, 0x06, 0x29,
-          USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "IBM FME", 0x0374, 0x08, 0x00, 0x5a,
-	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "IBM FME", 0x0374, 0x00, 0x04, 0xac,
-          USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "Hypertec Ethernet", 0x01c0, 0x00, 0x40, 0x4c, 0 },
+    { "IBM CCAE", 0x0ff0, 0x08, 0x00, 0x5a,
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+    { "IBM CCAE", 0x0ff0, 0x00, 0x04, 0xac,
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+    { "IBM CCAE", 0x0ff0, 0x00, 0x06, 0x29,
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+    { "IBM FME", 0x0374, 0x08, 0x00, 0x5a,
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+    { "IBM FME", 0x0374, 0x00, 0x04, 0xac,
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+    { "I-O DATA PCLA/T", 0x0ff0, 0x00, 0xa0, 0xb0, 0 },
     { "Katron PE-520", 0x0110, 0x00, 0x40, 0xf6, 0 },
     { "Kingston KNE-PCM/x", 0x0ff0, 0x00, 0xc0, 0xf0,
-	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "Kingston KNE-PCM/x", 0x0ff0, 0xe2, 0x0c, 0x0f,
-	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+    { "Kingston KNE-PC2", 0x0180, 0x00, 0xc0, 0xf0, 0 },
     { "Longshine LCS-8534", 0, 0x08, 0x00, 0x00, 0 },
     { "Maxtech PCN2000", 0x5000, 0x00, 0x00, 0xe8, 0 },
     { "NDC Instant-Link", 0x003a, 0x00, 0x80, 0xc6, 0 },
     { "NE2000 Compatible", 0x0ff0, 0x00, 0xa0, 0x0c, 0 },
     { "Network General Sniffer", 0x0ff0, 0x00, 0x00, 0x65,
-	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "Panasonic VEL211", 0x0ff0, 0x00, 0x80, 0x45, 
-          USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+      USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "PreMax PE-200", 0x07f0, 0x00, 0x20, 0xe0, 0 },
     { "RPTI EP400", 0x0110, 0x00, 0x40, 0x95, 0 },
     { "SCM Ethernet", 0x0ff0, 0x00, 0x20, 0xcb, 0 },
     { "Socket EA", 0x4000, 0x00, 0xc0, 0x1b,
-	  DELAY_OUTPUT | HAS_MISC_REG | USE_BIG_BUF }
+      DELAY_OUTPUT | HAS_MISC_REG | USE_BIG_BUF },
+    { "Volktek NPL-402CT", 0x0060, 0x00, 0x40, 0x05, 0 }
 };
 
 #define NR_INFO		(sizeof(hw_info)/sizeof(hw_info_t))
@@ -244,10 +261,9 @@ static dev_link_t *pcnet_attach(void)
     link->io.NumPorts1 = link->io.NumPorts2 = 16;
     link->io.Attributes2 = IO_DATA_PATH_WIDTH_16;
     link->io.IOAddrLines = 5;
-    link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
+    link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
     link->irq.IRQInfo2 = irq_mask;
-    link->irq.Handler = &ei_interrupt;
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
@@ -547,10 +563,11 @@ static void pcnet_config(dev_link_t *link)
 	    (buf[1] == PRODID_LINKSYS_PCMLM28)) {
 	    multi = 1;
 	    link->io.NumPorts1 = 32;
-	    link->io.Attributes1 = IO_DATA_PATH_WIDTH_16;
+	    link->io.Attributes1 = IO_DATA_PATH_WIDTH_AUTO;
 	    link->io.NumPorts2 = 8;
+	    link->io.Attributes2 = IO_DATA_PATH_WIDTH_8;
 	    link->irq.Attributes =
-		IRQ_TYPE_DYNAMIC_SHARING|IRQ_FIRST_SHARED|IRQ_HANDLE_PRESENT;
+		IRQ_TYPE_DYNAMIC_SHARING|IRQ_FIRST_SHARED;
 	}
     }
     
@@ -595,6 +612,10 @@ static void pcnet_config(dev_link_t *link)
     
     i = CardServices(RequestIRQ, link->handle, &link->irq);
     CS_EXIT_TEST(i, RequestIRQ, config_failed);
+    if (multi) {
+	link->conf.Attributes |= CONF_ENABLE_SPKR;
+	link->conf.Status = CCSR_AUDIO_ENA;
+    }
     if (multi && (manfid == MANFID_IBM))
 	/* For IBM Home and Away Card */
 	link->conf.ConfigIndex |= 0x10;
@@ -624,7 +645,7 @@ static void pcnet_config(dev_link_t *link)
 	goto config_undo;
     }
 
-    info->flags = hw_info->flags;
+    info->flags = hw_info->flags | (multi ? IS_MULTIFN : 0);
     /* Check for user overrides */
     if (delay_output)
 	info->flags |= DELAY_OUTPUT;
@@ -818,8 +839,9 @@ static void set_misc_reg(struct device *dev)
 
 static int pcnet_open(struct device *dev)
 {
+    pcnet_dev_t *info = (pcnet_dev_t *)dev;
     dev_link_t *link;
-
+    
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
 	printk(KERN_DEBUG "pcnet_open('%s')\n", dev->name);
@@ -834,7 +856,9 @@ static int pcnet_open(struct device *dev)
     MOD_INC_USE_COUNT;
 
     set_misc_reg(dev);
-
+    REQUEST_IRQ(dev->irq, ei_interrupt,
+		((info->flags & IS_MULTIFN) ? SA_SHIRQ : 0),
+		dev_info, dev_info);
     return ei_open(dev);
 } /* pcnet_open */
 
@@ -854,6 +878,8 @@ static int pcnet_close(struct device *dev)
     if (link == NULL)
 	return -ENODEV;
 
+    FREE_IRQ(dev->irq, dev_info);
+    
     link->open--; dev->start = 0;
     if (link->state & DEV_STALE_CONFIG) {
 	link->release.expires = RUN_AT(HZ/20);
@@ -880,7 +906,7 @@ static void pcnet_reset_8390(struct device *dev)
 
     ei_status.txing = ei_status.dmaing = 0;
 
-    outb(inb_p(nic_base + PCNET_RESET), nic_base + PCNET_RESET);
+    outb(inb(nic_base + PCNET_RESET), nic_base + PCNET_RESET);
 
     for (i = 0; i < 100; i++) {
 	if ((inb_p(nic_base+EN0_ISR) & ENISR_RESET) != 0)
