@@ -38,7 +38,11 @@
 #include "drivers/scsi/qlogicfas.h"
 #define QLOGIC QLOGICFAS
 #define qlogic_preset qlogicfas_preset
+#if (LINUX_VERSION_CODE >= VERSION(2,1,18))
+#define qlogic_reset(h) qlogicfas_reset(h, 0)
+#else
 #define qlogic_reset qlogicfas_reset
+#endif
 #else
 #include "drivers/scsi/qlogic.h"
 #endif
@@ -54,8 +58,9 @@ extern void qlogic_preset(int port, int irq);
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"qlogic_cs.c 1.42 1996/12/23 06:32:47 (David Hinds)";
+"qlogic_cs.c 1.46 1997/03/16 20:03:23 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -65,6 +70,8 @@ static char *version =
 /* Bit map of interrupts to choose from */
 static u_long irq_mask = 0xdeb8;
 
+MODULE_PARM(irq_mask, "i");
+
 /*====================================================================*/
 
 typedef struct scsi_info_t {
@@ -194,7 +201,7 @@ static void qlogic_config(dev_link_t *link)
     tuple_t tuple;
     cisparse_t parse;
     int i;
-    u_short tuple_data[32];
+    u_char tuple_data[64];
 #ifdef GET_SCSI_INFO
     Scsi_Device *dev;
     dev_node_t **tail, *node;
@@ -237,8 +244,12 @@ static void qlogic_config(dev_link_t *link)
     } while (0);
 
     /* Configure card */
+#if (LINUX_VERSION_CODE >= VERSION(2,1,23))
+    driver_template.module = &__this_module;
+#else
+    driver_template.usage_count = &MOD_USE_COUNT;
+#endif
     link->state |= DEV_CONFIG;
-    driver_template.usage_count = &mod_use_count_;
 
     do {
 
@@ -253,7 +264,7 @@ static void qlogic_config(dev_link_t *link)
 	    link->io.BasePort1 = parse.cftable_entry.io.win[0].base;
 	    if (link->io.BasePort1 != 0) {
 		i = CardServices(RequestIO, link->handle, &link->io);
-		if (i == CS_SUCCESS) break;
+		if (i != CS_IN_USE) break;
 	    }
 	    i = CardServices(GetNextTuple, handle, &tuple);
 	}
@@ -351,7 +362,11 @@ static void qlogic_release(u_long arg)
 	printk(KERN_DEBUG "qlogic_release(0x%p)\n", link);
 #endif
 
+#if (LINUX_VERSION_CODE < VERSION(2,1,23))
     if (*driver_template.usage_count != 0) {
+#else
+    if (driver_template.module->usecount != 0) {
+#endif
 #ifdef PCMCIA_DEBUG
 	if (pc_debug)
 	    printk(KERN_DEBUG "qlogic_cs: release postponed, "
