@@ -66,6 +66,9 @@ irq_info_t irq_table[16] = { { 0, 0, 0 }, /* etc */ };
 static int probe_io = 1;
 static int probe_mem = 0;
 
+MODULE_PARM(probe_io, "i");
+MODULE_PARM(probe_mem, "i");
+
 /*======================================================================
 
     These functions manage the database of allocated memory blocks.
@@ -239,7 +242,8 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
 	if (b[i] > b[hole]) hole = i;
     kfree(b);
 
-    printk(KERN_INFO "PCMCIA IO port probe:");
+    printk(KERN_INFO "cs: IO port probe 0x%04x-0x%04x:",
+	   base, base+num-1);
     bad = any = 0;
     for (i = base; i < base+num; i += 8) {
 	if (check_region(i, 8) != 0)
@@ -261,7 +265,7 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
 	}
     }
     if (bad) {
-	if ((bad == base) && (i == base+num))
+	if ((num > 16) && (bad == base) && (i == base+num))
 	    printk(" nothing: probe failed.\n");
 	else {
 	    sub_interval(&io_db, bad, i-bad);
@@ -409,7 +413,7 @@ int try_irq(irq_req_t *req, int irq, int specific)
     else {
 	if ((info->Attributes & RES_RESERVED) && !specific)
 	    return CS_IN_USE;
-	if (REQUEST_IRQ(irq, NULL, 0, "bogus") == -EBUSY)
+	if (REQUEST_IRQ(irq, NULL, 0, "bogus", NULL) == -EBUSY)
 	    return CS_IN_USE;
 	switch (req->Attributes & IRQ_TYPE) {
 	case IRQ_TYPE_EXCLUSIVE:
