@@ -31,8 +31,9 @@
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"serial_cs.c 1.78 1996/12/22 03:49:53 (David Hinds)";
+"serial_cs.c 1.81 1997/03/16 20:09:13 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -45,6 +46,9 @@ static u_long irq_mask = 0xdeb8;
 /* Enable the speaker? */
 static int do_sound = 1;
 
+MODULE_PARM(irq_mask, "i");
+MODULE_PARM(do_sound, "i");
+
 /*====================================================================*/
 
 /* Table of multifunction card ID's */
@@ -64,7 +68,8 @@ static multi_id_t multi_id[] = {
     { MANFID_OSITECH, PRODID_OSITECH_JACK_288, 1, 1 },
     { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_VARIOUS, 1, 1 },
     { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_EM3288, 1, 1 },
-    { MANFID_QUATECH, PRODID_QUATECH_DUAL_RS232, 2, 0 }
+    { MANFID_QUATECH, PRODID_QUATECH_DUAL_RS232, 2, 0 },
+    { MANFID_SOCKET, PRODID_SOCKET_DUAL_RS232, 2, 0 }
 };
 #define MULTI_COUNT (sizeof(multi_id)/sizeof(multi_id_t))
 
@@ -332,7 +337,6 @@ static int slave_config(dev_link_t *link)
     ioaddr_t port;
     int i;
 
-    printk("slave_config()\n");
     /* If the card is already configured, look up the port and irq */
     i = CardServices(GetConfigurationInfo, handle, &config);
     if ((i == CS_SUCCESS) &&
@@ -356,23 +360,44 @@ static int dual_config(dev_link_t *link)
     u_char buf[256];
     cisparse_t parse;
     cistpl_cftable_entry_t *cf = &parse.cftable_entry;
-    int i;
+    int i, base2;
 
-    link->io.NumPorts1 = 16;
     tuple.TupleData = (cisdata_t *)buf;
     tuple.TupleOffset = 0; tuple.TupleDataMax = 255;
     tuple.Attributes = 0;
     tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
+
+    /* First, look for a generic 16-port window */
+    link->io.NumPorts1 = 16;
     i = first_tuple(handle, &tuple, &parse);
     while (i == CS_SUCCESS) {
 	if ((cf->io.nwin == 1) && (cf->io.win[0].len == 16)) {
 	    link->conf.ConfigIndex = cf->index;
 	    link->io.BasePort1 = cf->io.win[0].base;
 	    i = CardServices(RequestIO, link->handle, &link->io);
+	    base2 = link->io.BasePort1 + 8;
 	    if (i == CS_SUCCESS) break;
 	}
 	i = next_tuple(handle, &tuple, &parse);
     }
+
+    /* If that didn't work, look for two windows */
+    if (i != CS_SUCCESS) {
+	link->io.NumPorts1 = link->io.NumPorts2 = 8;
+	i = first_tuple(handle, &tuple, &parse);
+	while (i == CS_SUCCESS) {
+	    if (cf->io.nwin == 2) {
+		link->conf.ConfigIndex = cf->index;
+		link->io.BasePort1 = cf->io.win[0].base;
+		link->io.BasePort2 = cf->io.win[1].base;
+		i = CardServices(RequestIO, link->handle, &link->io);
+		base2 = link->io.BasePort2;
+		if (i == CS_SUCCESS) break;
+	    }
+	    i = next_tuple(handle, &tuple, &parse);
+	}
+    }
+    
     if (i != CS_SUCCESS) {
 	cs_error(RequestIO, i);
 	return -1;
@@ -388,9 +413,10 @@ static int dual_config(dev_link_t *link)
 	cs_error(RequestConfiguration, i);
 	return -1;
     }
-
+    
     setup_serial(info, link->io.BasePort1, link->irq.AssignedIRQ);
-    setup_serial(info, link->io.BasePort1+8, link->irq.AssignedIRQ);
+    setup_serial(info, base2, link->irq.AssignedIRQ);
+
     return 0;
 }
 
