@@ -33,8 +33,9 @@
 */
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
-"skeleton.c 1.34 1996/12/21 19:30:58 (David Hinds)";
+"skeleton.c 1.36 1997/03/16 20:06:32 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -45,6 +46,8 @@ static char *version =
 /* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
 static u_long irq_mask = 0xdeb8;
 
+MODULE_PARM(irq_mask, "i");
+
 /*====================================================================*/
 
 /*
@@ -419,9 +422,11 @@ static void skeleton_release(u_long arg)
        is actually closed.
     */
     if (link->open) {
+#ifdef PCMCIA_DEBUG
 	if (pc_debug)
 	    printk(KERN_DEBUG "skeleton_cs: release postponed, '%s' "
 		   "still open\n", link->dev->dev_name);
+#endif
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
