@@ -51,6 +51,7 @@
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static const char *version =
 "smc91c92_cs.c 0.09 1996/8/4 Donald Becker, becker@cesdis.gsfc.nasa.gov.\n";
 #endif
@@ -69,6 +70,9 @@ static int if_port = 1;
 /* Bit map of interrupts to choose from. */
 static u_long irq_mask = 0xdeb8;
 
+MODULE_PARM(if_port, "i");
+MODULE_PARM(irq_mask, "i");
+
 /* Operational parameter that usually are not changed. */
 
 /* Do you want to use 32 bit xfers?  This should work on all chips,
@@ -95,7 +99,7 @@ static dev_link_t *dev_list = NULL;
 struct smc_private {
     u_short			manfid;
     u_short			cardid;
-    struct enet_statistics	stats;
+    struct net_device_stats	stats;
     dev_node_t			node;
     struct sk_buff		*saved_skb;
     int				packets_waiting;
@@ -223,7 +227,7 @@ static int smc91c92_close(struct device *dev);
 static int smc_start_xmit(struct sk_buff *skb, struct device *dev);
 static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs);
 static void smc_rx(struct device *dev);
-static struct enet_statistics *smc91c92_get_stats(struct device *dev);
+static struct net_device_stats *smc91c92_get_stats(struct device *dev);
 static void set_rx_mode(struct device *dev);
 static int s9k_config(struct device *dev, struct ifmap *map);
 static void smc_reset(struct device *dev);
@@ -1057,6 +1061,9 @@ static void smc_hardware_send_packet( struct device * dev )
 	return;
     }
     
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+    lp->stats.tx_bytes += skb->len;
+#endif
     /* The card should use the just-allocated buffer. */
     outb( packet_no, ioaddr + PNR_ARR );
     /* point to the beginning of the packet */
@@ -1128,7 +1135,8 @@ static int smc_start_xmit(struct sk_buff *skb, struct device *dev)
 	dev->trans_start = jiffies;
 	dev->tbusy = 0;
     }
-    
+
+#if (LINUX_VERSION_CODE < VERSION(2,1,25))
     /* Sanity check the queue-layer request. */
     if (skb == NULL) {
 	dev_tint(dev);
@@ -1136,6 +1144,7 @@ static int smc_start_xmit(struct sk_buff *skb, struct device *dev)
     }
     if (skb->len <= 0)
 	return 0;
+#endif
     
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 2)
@@ -1263,24 +1272,19 @@ static void smc_eph_irq(struct device *dev)
     outw( inw(ioaddr + TCR) | TCR_ENABLE, ioaddr + TCR);
     /* Ack potential Link Error status. */
     SMC_SELECT_BANK(1);
-    if ((dev->if_port == 2)  && (ephs & 0x4000)) {
-	dev->if_port = 1;
-	printk(KERN_INFO "%s: Link beat returned -- switching to"
-	       " 10baseT.\n", dev->name);
-    } else if ((dev->if_port == 1) && (ephs & 0x4000) == 0) {
-	dev->if_port = 2;
-	printk(KERN_INFO "%s: Link beat lost -- switching to"
-	       " 10base2.\n", dev->name);
-    } else goto skip;
-    outw(CFG_NO_WAIT | CFG_16BIT | CFG_STATIC |
-	 (lp->manfid == MANFID_OSITECH ?
-	  (CFG_IRQ_SEL_1 | CFG_IRQ_SEL_0) : 0) |
-	 (dev->if_port == 2 ? CFG_AUI_SELECT : 0),
-	 ioaddr + CONFIG);
-    if (lp->manfid == MANFID_OSITECH)
-	outb(dev->if_port == 2 ? OSI_AUI_PWR : 0,
-	     ioaddr - 0x10 + OSITECH_AUI_CTL);
-    skip:
+    if ((dev->if_port == 1) && !(ephs & 0x4000)) {
+	printk(KERN_INFO "%s: Link beat lost\n", dev->name);
+	if (lp->stats.rx_packets < 10) {
+	    printk(KERN_INFO "%s: switching to 10base2.\n", dev->name);
+	    dev->if_port = 2;
+	    outw(CFG_NO_WAIT | CFG_16BIT | CFG_STATIC | CFG_AUI_SELECT |
+		 (lp->manfid == MANFID_OSITECH ?
+		  (CFG_IRQ_SEL_1 | CFG_IRQ_SEL_0) : 0),
+		 ioaddr + CONFIG);
+	    if (lp->manfid == MANFID_OSITECH)
+		outb(OSI_AUI_PWR, ioaddr - 0x10 + OSITECH_AUI_CTL);
+	}
+    }
 
     /* Clear a link error interrupt. */
     outw( CTL_AUTO_RELEASE | 0x0000, ioaddr + CONTROL);
@@ -1470,6 +1474,9 @@ static void smc_rx(struct device *dev)
 	skb->dev = dev;
 	netif_rx(skb);
 	lp->stats.rx_packets++;
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+	lp->stats.rx_bytes += skb->len;
+#endif
 	if (rx_status & RS_MULTICAST)
 	    lp->stats.multicast++;
     } else {
@@ -1489,7 +1496,7 @@ static void smc_rx(struct device *dev)
 
 /*====================================================================*/
 
-static struct enet_statistics *smc91c92_get_stats(struct device *dev)
+static struct net_device_stats *smc91c92_get_stats(struct device *dev)
 {
     struct smc_private *lp = (struct smc_private *)dev->priv;
     /* Nothing to update - the 91c92 is a pretty primative chip. */
