@@ -34,7 +34,7 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static const char *version =
-"tcic.c 1.78 1996/12/21 19:31:00 (David Hinds)";
+"tcic.c 1.80 1997/03/16 20:07:08 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -68,6 +68,16 @@ static int cycle_time = 70;
 /* Selects asynchronous clocking */
 static int async_clock = 0;
 
+MODULE_PARM(tcic_base, "i");
+MODULE_PARM(ignore, "i");
+MODULE_PARM(do_scan, "i");
+MODULE_PARM(irq_mask, "i");
+MODULE_PARM(cs_irq, "i");
+MODULE_PARM(poll_interval, "i");
+MODULE_PARM(poll_quick, "i");
+MODULE_PARM(cycle_time, "i");
+MODULE_PARM(async_clock, "i");
+
 /*====================================================================*/
 
 static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs);
@@ -206,12 +216,12 @@ static u_long try_irq(int irq)
 {
     u_short cfg;
 
-    if (REQUEST_IRQ(irq, irq_count, 0, "irq scan") != 0)
+    if (REQUEST_IRQ(irq, irq_count, 0, "irq scan", NULL) != 0)
 	return -1;
     irq_hits = 0;
     busy_loop(HZ/50);
     if (irq_hits) {
-	FREE_IRQ(irq);
+	FREE_IRQ(irq, NULL);
 	return -1;
     }
 
@@ -222,7 +232,7 @@ static u_long try_irq(int irq)
     tcic_setb(TCIC_ICSR, TCIC_ICSR_ERR | TCIC_ICSR_JAM);
 
     busy_loop(HZ/50);
-    FREE_IRQ(irq);
+    FREE_IRQ(irq, NULL);
 
     /* Turn off interrupts */
     tcic_setb(TCIC_IENA, TCIC_IENA_CFG_OFF);
@@ -413,12 +423,12 @@ int tcic_init(void)
 	if (cs_irq == 0) {
 	    for (i = 15; i > 0; i--)
 		if ((i != 12) && (mask & (1 << i)) &&
-		    (REQUEST_IRQ(i, tcic_interrupt, 0, "tcic") == 0))
+		    (REQUEST_IRQ(i, tcic_interrupt, 0, "tcic", NULL) == 0))
 		    break;
 	    cs_irq = i;
 	}
 	else {
-	    if (REQUEST_IRQ(cs_irq, tcic_interrupt, 0, "tcic") == 0)
+	    if (REQUEST_IRQ(cs_irq, tcic_interrupt, 0, "tcic", NULL) == 0)
 		cs_irq = 0;
 	}
 	if (cs_irq == 0) poll_interval = 100;
@@ -443,7 +453,7 @@ int tcic_init(void)
 	printk(KERN_NOTICE "tcic: register_ss_entry() failed\n");
 	release_region(tcic_base, 16);
 	if (cs_irq != 0)
-	    FREE_IRQ(cs_irq);
+	    FREE_IRQ(cs_irq, NULL);
 	return -ENODEV;
     }
 
@@ -461,7 +471,7 @@ static void tcic_finish(void)
     cli();
     if (cs_irq != 0) {
 	tcic_aux_setw(TCIC_AUX_SYSCFG, TCIC_SYSCFG_AUTOBUSY|0x0a00);
-	FREE_IRQ(cs_irq);
+	FREE_IRQ(cs_irq, NULL);
     }
     if (timer_pending)
 	del_timer(&poll_timer);
