@@ -120,10 +120,6 @@
  * reorganisation.
  * Loeke Brederveld <lbrederv@wavelan.com> from Lucent has given me
  * much needed informations on the Wavelan hardware.
- *
- * Yongguang Zhang <ygz@isl.hrl.hac.com> send me a patch for enabling
- * multicast in the old pcmcia driver that I included in this driver
- * (with some minor changes).
  */
 
 /* The original copyrights and litteratures mention others names and
@@ -185,6 +181,7 @@
  *	Peter Storey,
  *	Jean Tourrilhes <jt@hplb.hpl.hp.com>,
  *	Girish Welling (welling@paul.rutgers.edu)
+ *	Clark Woodworth <clark@hiway1.exit109.com>
  *	Yongguang Zhang <ygz@isl.hrl.hac.com>...
  */
 
@@ -258,16 +255,27 @@
  *	- Revisited multicast code (it was mostly wrong).
  *	- protect code in wv_82593_reconfig with dev->tbusy (oups !)
  *
- * Changes made in sixth release (2.9.1) :
- * -------------------------------------
+ * Changes made in sixth release (2.9.1a) :
+ * --------------------------------------
  *	- Change the detection code for multi manufacturer code support
  *	- Correct bug (hang kernel) in init when we were "rejecting" a card 
  *
- * Changes made in this release (2.9.2) :
- * ------------------------------------
+ * Changes made in seventh release (2.9.1b) :
+ * ----------------------------------------
  *	- Update to wireless extensions changes
  *	- Silly bug in card initial configuration (psa_conf_status)
  *
+ * Changes made in eigth release :
+ * -----------------------------
+ *	- Small bug in debug code (probably not the last one...)
+ *	- 1.2.13 support (thanks to Clark Woodworth)
+ *
+ * Changes made in this release (2.9.2b) :
+ * -------------------------------------
+ *	- Level threshold is now a standard wireless extension (version 4 !)
+ *	- modules parameters types for kernel > 2.1.17
+ *	- updated man page
+ *
  * Wishes & dreams :
  * ---------------
  *	- Encryption stuff
@@ -302,6 +310,10 @@
 
 #ifdef HAS_WIRELESS_EXTENSIONS
 #include <linux/wireless.h>		/* Wireless extensions */
+#if WIRELESS_EXT < 4
+#warning "You need to upgrade your version of Wireless Extensions."
+#undef WIRELESS_EXT
+#endif
 #endif
 
 /* Pcmcia headers that we need */
@@ -364,7 +376,7 @@
 /************************ CONSTANTS & MACROS ************************/
 
 #ifdef DEBUG_VERSION_SHOW
-static const char *version = "wavelan_cs.c : v12 (wireless extensions) 2/12/96\n";
+static const char *version = "wavelan_cs.c : v15 (wireless extensions) 12/2/97\n";
 #endif
 
 /* Watchdog temporisation */
@@ -380,19 +392,11 @@ static const char *version = "wavelan_cs.c : v12 (wireless extensions) 2/12/96\n
 #define SIOCSIPHISTO	SIOCDEVPRIVATE + 6	/* Set histogram ranges */
 #define SIOCGIPHISTO	SIOCDEVPRIVATE + 7	/* Get histogram values */
 
-/* ----------------------- VERSION SUPPORT ----------------------- */
-
-/* This ugly patch is needed to cope with old version of the kernel */
-#ifndef copy_from_user
-#define copy_from_user	memcpy_fromfs
-#define copy_to_user	memcpy_tofs
-#endif
-
 /****************************** TYPES ******************************/
 
 /* Shortcuts */
 typedef struct device		device;
-typedef struct enet_statistics	en_stats;
+typedef struct net_device_stats	en_stats;
 typedef struct iw_statistics	iw_stats;
 typedef struct iw_quality	iw_qual;
 typedef struct iw_freq		iw_freq;
@@ -591,9 +595,6 @@ static volatile int	wv_wait_completed = 0;
 /*
  * Parameters that can be set with 'insmod'
  * The exact syntax is 'insmod wavelan_cs.o <var>=<value>'
- *
- * For the long run, it would be probably better that you change the values
- * here and recompile...
  */
 
 /* Bit map of interrupts to choose from */
@@ -603,11 +604,10 @@ static const u_long	irq_mask = 0xdeb8;
 /* Shared memory speed, in ns */
 static const int	mem_speed = 0;
 
-/*
- * NOTE : gathersnr has been removed, has with the histogram mecanisms it is
- * set through the ioctl.
- *	  all debugs have been removed from here. If you shout at me (and if
- * I have time), I will think about it...
- */
+/* New module interface */
+#if (LINUX_VERSION_CODE > VERSION(2,1,17))
+MODULE_PARM(irq_mask, "1l");
+MODULE_PARM(mem_speed, "1i");
+#endif	/* > 2.1.17 */
 
 #endif	/* WAVELAN_CS_H */
