@@ -129,6 +129,7 @@ int xinw_debug = 0;
 */
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static char *version =
 "xircnw_cs.c 0.2.0 1996/05/22 15:05:51 (John Markus Bjrndalen)\n";
 #endif
@@ -158,6 +159,11 @@ static int mem_speed = 0;
 /* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
 static u_long irq_mask = 0xdeb8;
 
+MODULE_PARM(domain, "i");
+MODULE_PARM(scramble_key, "i");
+MODULE_PARM(mem_speed, "i");
+MODULE_PARM(irq_mask, "i");
+
 /*====================================================================*/
 
 /*
@@ -186,7 +192,7 @@ static int xinw_open(struct device *dev);
 static int xinw_start_xmit(struct sk_buff *skb, struct device *dev);
 static void xinw_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs);
 static void update_stats(struct device *dev);
-static struct enet_statistics *xinw_get_stats(struct device *dev);
+static struct net_device_stats *xinw_get_stats(struct device *dev);
 static int xinw_rx(struct device *dev);
 static int xinw_close(struct device *dev);
 
@@ -237,12 +243,12 @@ static dev_link_t *dev_list = NULL;
 typedef struct xircom_private {
     dev_node_t	node;
     u_char      *ramBase;
-    int         timeoutCounter;
-    struct enet_statistics stats;
+    int        	timeoutCounter;
+    struct net_device_stats stats;
 } xircom_private;
 
 #ifdef XIRCNW_STATS
-static struct enet_statistics *xircnw_get_stats(struct device *dev);
+static struct net_device_stats *xircnw_get_stats(struct device *dev);
 #endif
 
 /*
@@ -849,6 +855,9 @@ static int xircnw_hw_xmit(unsigned char* data, int len, struct device* dev)
 	return 1;
     }
 
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+    priv->stats.tx_bytes += len;
+#endif
 
     if (xinw_debug > 4) {
 	printk(KERN_DEBUG "Transmitting with SPCQ %x SPU %x LIF %x ISPLQ %x\n",
@@ -921,17 +930,15 @@ static int xinw_start_xmit(struct sk_buff *skb, struct device *dev)
 	dev->tbusy = 0;
     }
 
-    /* Sending a NULL skb means some higher layer thinks we've missed an
-       tx-done interrupt. Caution: dev_tint() handles the cli()/sti()
-       itself. */
+#if (LINUX_VERSION_CODE < VERSION(2,1,25))
     if (skb == NULL) {
         dev_tint(dev);
 	return 0;
     }
-   
     if (skb->len <= 0)
 	return 0;
-
+#endif
+    
     /* Block a timer-based transmit from overlapping. This could 
        better be done with atomic_swap(1, dev->tbusy, but set_bit()
        works as well */
@@ -1101,7 +1108,7 @@ static void xinw_interrupt IRQ(int irq, void* dev_id, struct pt_regs *regs)
     return;
 } /* xinw_interrupt */
 
-static struct enet_statistics *xinw_get_stats(struct device *dev)
+static struct net_device_stats *xinw_get_stats(struct device *dev)
 {
     xircom_private *priv = (xircom_private*)dev->priv;
     unsigned long flags;
@@ -1252,6 +1259,9 @@ static int xinw_rx(struct device *dev)
 	    printk(KERN_DEBUG "Packet reception ok\n");
       
 	priv->stats.rx_packets++;
+#if (LINUX_VERSION_CODE >= VERSION(2,1,25))
+	priv->stats.rx_bytes += skb->len;
+#endif
     }
     return 0;
 }
